@@ -115,6 +115,7 @@ RZ_API RzBinRelocStorage *rz_bin_reloc_storage_new(RZ_OWN RzPVector /*<RzBinRelo
 			rz_pvector_push(&target_sorter, reloc);
 		}
 	}
+	ret->relocs_free = relocs->v.free_user;
 	relocs->v.free = NULL; // ownership of relocs transferred
 	rz_pvector_free(relocs);
 	rz_pvector_sort(&sorter, reloc_cmp, NULL);
@@ -132,8 +133,10 @@ RZ_API void rz_bin_reloc_storage_free(RzBinRelocStorage *storage) {
 	if (!storage) {
 		return;
 	}
-	for (size_t i = 0; i < storage->relocs_count; i++) {
-		rz_bin_reloc_free(storage->relocs[i]);
+	if (storage->relocs_free) {
+		for (size_t i = 0; i < storage->relocs_count; i++) {
+			storage->relocs_free(storage->relocs[i]);
+		}
 	}
 	free(storage->relocs);
 	free(storage->target_relocs);
@@ -184,17 +187,17 @@ RZ_IPI void rz_bin_object_free(RzBinObject *o) {
 		return;
 	}
 	free(o->regstate);
-	ht_pp_free(o->glue_to_class_field);
-	ht_pp_free(o->glue_to_class_method);
-	ht_pp_free(o->name_to_class_object);
-	ht_pp_free(o->import_name_symbols);
+	ht_sp_free(o->glue_to_class_field);
+	ht_sp_free(o->glue_to_class_method);
+	ht_sp_free(o->name_to_class_object);
+	ht_sp_free(o->import_name_symbols);
 	ht_up_free(o->vaddr_to_class_method);
 	rz_bin_info_free(o->info);
 	rz_bin_reloc_storage_free(o->relocs);
 	rz_bin_source_line_info_free(o->lines);
 	rz_bin_string_database_free(o->strings);
 	rz_pvector_free(o->classes);
-	rz_list_free(o->entries);
+	rz_pvector_free(o->entries);
 	rz_pvector_free(o->fields);
 	rz_pvector_free(o->imports);
 	rz_pvector_free(o->libs);
@@ -220,7 +223,7 @@ RZ_IPI void rz_bin_object_free(RzBinObject *o) {
  */
 RZ_API RZ_BORROW RzBinClass *rz_bin_object_find_class(RZ_NONNULL RzBinObject *o, RZ_NONNULL const char *name) {
 	rz_return_val_if_fail(o && name, NULL);
-	return ht_pp_find(o->name_to_class_object, name, NULL);
+	return ht_sp_find(o->name_to_class_object, name, NULL);
 }
 
 static RzBinClass *bin_class_new(RzBinObject *o, const char *name, const char *super, ut64 address) {
@@ -229,7 +232,7 @@ static RzBinClass *bin_class_new(RzBinObject *o, const char *name, const char *s
 		return NULL;
 	}
 
-	c->name = strdup(name);
+	c->name = rz_str_dup(name);
 	c->super = rz_str_dup(super);
 	c->methods = rz_list_newf((RzListFree)rz_bin_symbol_free);
 	c->fields = rz_list_newf((RzListFree)rz_bin_class_field_free);
@@ -298,10 +301,10 @@ static int bin_compare_class_field(RzBinClassField *a, RzBinClassField *b, void
 RZ_API RZ_BORROW RzBinClass *rz_bin_object_add_class(RZ_NONNULL RzBinObject *o, RZ_NONNULL const char *name, RZ_NULLABLE const char *super, ut64 vaddr) {
 	rz_return_val_if_fail(o && RZ_STR_ISNOTEMPTY(name), NULL);
 
-	RzBinClass *oclass = ht_pp_find(o->name_to_class_object, name, NULL);
+	RzBinClass *oclass = ht_sp_find(o->name_to_class_object, name, NULL);
 	if (oclass) {
 		if (super && !oclass->super) {
-			oclass->super = strdup(super);
+			oclass->super = rz_str_dup(super);
 		}
 		if (oclass->addr == UT64_MAX) {
 			oclass->addr = vaddr;
@@ -316,7 +319,7 @@ RZ_API RZ_BORROW RzBinClass *rz_bin_object_add_class(RZ_NONNULL RzBinObject *o,
 
 	rz_pvector_push(o->classes, oclass);
 	rz_pvector_sort(o->classes, (RzPVectorComparator)bin_compare_class, NULL);
-	ht_pp_insert(o->name_to_class_object, name, oclass);
+	ht_sp_insert(o->name_to_class_object, name, oclass);
 	return oclass;
 }
 
@@ -336,7 +339,7 @@ RZ_API RzBinSymbol *rz_bin_object_find_method(RZ_NONNULL RzBinObject *o, RZ_NONN
 	if (!key) {
 		return NULL;
 	}
-	RzBinSymbol *sym = (RzBinSymbol *)ht_pp_find(o->glue_to_class_method, key, NULL);
+	RzBinSymbol *sym = (RzBinSymbol *)ht_sp_find(o->glue_to_class_method, key, NULL);
 	free(key);
 	return sym;
 }
@@ -401,7 +404,7 @@ RZ_API RZ_BORROW RzBinSymbol *rz_bin_object_add_method(RZ_NONNULL RzBinObject *o
 
 	char *key = rz_str_newf(RZ_BIN_FMT_CLASS_HT_GLUE, klass, method);
 	if (key) {
-		ht_pp_insert(o->glue_to_class_method, key, symbol);
+		ht_sp_insert(o->glue_to_class_method, key, symbol);
 		free(key);
 	}
 
@@ -428,7 +431,7 @@ RZ_API RzBinClassField *rz_bin_object_find_field(RZ_NONNULL RzBinObject *o, RZ_N
 	if (!key) {
 		return NULL;
 	}
-	RzBinClassField *sym = (RzBinClassField *)ht_pp_find(o->glue_to_class_field, key, NULL);
+	RzBinClassField *sym = (RzBinClassField *)ht_sp_find(o->glue_to_class_field, key, NULL);
 	free(key);
 	return sym;
 }
@@ -478,7 +481,7 @@ RZ_API RZ_BORROW RzBinClassField *rz_bin_object_add_field(RZ_NONNULL RzBinObject
 	rz_list_add_sorted(c->fields, field, (RzListComparator)bin_compare_class_field, NULL);
 	char *key = rz_str_newf(RZ_BIN_FMT_CLASS_HT_GLUE, klass, name);
 	if (key) {
-		ht_pp_insert(o->glue_to_class_field, key, field);
+		ht_sp_insert(o->glue_to_class_field, key, field);
 		free(key);
 	}
 	return field;
@@ -569,7 +572,7 @@ RZ_API RzBinSymbol *rz_bin_object_get_symbol_of_import(RzBinObject *o, RzBinImpo
 	if (!o->import_name_symbols) {
 		return NULL;
 	}
-	return ht_pp_find(o->import_name_symbols, imp->name, NULL);
+	return ht_sp_find(o->import_name_symbols, imp->name, NULL);
 }
 
 RZ_API RzBinVirtualFile *rz_bin_object_get_virtual_file(RzBinObject *o, const char *name) {
@@ -648,8 +651,7 @@ RZ_API ut64 rz_bin_object_get_vaddr(RzBinObject *o, ut64 paddr, ut64 vaddr) {
 /**
  * \brief Return the \p RzBinAddr structure representing the special symbol \p sym
  */
-RZ_API const RzBinAddr *rz_bin_object_get_special_symbol(RzBinObject *o, RzBinSpecialSymbol sym) {
-	rz_return_val_if_fail(o, NULL);
+RZ_API RZ_BORROW const RzBinAddr *rz_bin_object_get_special_symbol(RZ_NULLABLE RzBinObject *o, RzBinSpecialSymbol sym) {
 	if (sym < 0 || sym >= RZ_BIN_SPECIAL_SYMBOL_LAST) {
 		return NULL;
 	}
@@ -657,9 +659,9 @@ RZ_API const RzBinAddr *rz_bin_object_get_special_symbol(RzBinObject *o, RzBinSp
 }
 
 /**
- * \brief Get list of \p RzBinAddr representing the entry points of the binary object.
+ * \brief Get pvector of \p RzBinAddr representing the entry points of the binary object.
  */
-RZ_API const RzList /*<RzBinAddr *>*/ *rz_bin_object_get_entries(RZ_NONNULL RzBinObject *obj) {
+RZ_API RZ_BORROW const RzPVector /*<RzBinAddr *>*/ *rz_bin_object_get_entries(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->entries;
 }
@@ -976,22 +978,16 @@ RZ_API RZ_OWN RzBinStrDb *rz_bin_string_database_new(RZ_NULLABLE RZ_OWN RzPVecto
 		return NULL;
 	}
 
-	void **it;
-	RzBinString *bstr;
-	db->pvec = rz_pvector_new((RzPVectorFree)rz_bin_string_free);
-	if (pvector) {
-		rz_pvector_foreach (pvector, it) {
-			bstr = *it;
-			rz_pvector_push(db->pvec, bstr);
-		}
-	}
-	db->phys = ht_up_new0();
-	db->virt = ht_up_new0();
+	db->pvec = pvector ? pvector : rz_pvector_new((RzPVectorFree)rz_bin_string_free);
+	db->phys = ht_up_new(NULL, NULL);
+	db->virt = ht_up_new(NULL, NULL);
 	if (!db->pvec || !db->phys || !db->virt) {
 		RZ_LOG_ERROR("rz_bin: Cannot allocate RzBinStrDb internal data structure.\n");
 		goto fail;
 	}
 
+	void **it;
+	RzBinString *bstr;
 	rz_pvector_foreach (pvector, it) {
 		bstr = *it;
 		if (!ht_up_update(db->phys, bstr->paddr, bstr)) {
