@@ -8,11 +8,11 @@ RZ_API bool rz_bin_dwarf_addr_get(
 	RZ_BORROW RZ_NONNULL const RzBinDwarfAddr *self,
 	RZ_BORROW RZ_NONNULL ut64 *address,
 	ut8 address_size, ut64 base, ut64 index) {
-	rz_return_val_if_fail(self && self->reader && address, false);
-	RzBinEndianReader *reader = self->reader;
-	RET_FALSE_IF_FAIL(reader);
-	rz_buf_seek(reader->buffer, (st64)(base + (index * address_size)), RZ_BUF_SET);
-	RET_FALSE_IF_FAIL(read_address(self->reader, address, address_size));
+	rz_return_val_if_fail(self && self->R && address, false);
+	RzBinEndianReader *R = self->R;
+	RET_FALSE_IF_FAIL(R);
+	R_seek(R, (st64)(base + (index * address_size)), RZ_BUF_SET);
+	RET_FALSE_IF_FAIL(R_read_address(self->R, address, address_size));
 	return true;
 }
 
@@ -20,21 +20,21 @@ RZ_API void rz_bin_dwarf_addr_free(RzBinDwarfAddr *self) {
 	if (!self) {
 		return;
 	}
-	RzBinEndianReader_free(self->reader);
+	R_free(self->R);
 	free(self);
 }
 
-RZ_API RZ_OWN RzBinDwarfAddr *rz_bin_dwarf_addr_new(RZ_OWN RZ_NONNULL RzBinEndianReader *reader) {
-	rz_return_val_if_fail(reader, NULL);
+RZ_API RZ_OWN RzBinDwarfAddr *rz_bin_dwarf_addr_new(RZ_OWN RZ_NONNULL RzBinEndianReader *R) {
+	rz_return_val_if_fail(R, NULL);
 	RzBinDwarfAddr *self = RZ_NEW0(RzBinDwarfAddr);
 	RET_NULL_IF_FAIL(self);
-	self->reader = reader;
+	self->R = R;
 	return self;
 }
 
 RZ_API RZ_OWN RzBinDwarfAddr *rz_bin_dwarf_addr_from_file(RZ_BORROW RZ_NONNULL RzBinFile *bf) {
 	rz_return_val_if_fail(bf, NULL);
-	RzBinEndianReader *r = RzBinEndianReader_from_file(bf, ".debug_addr", false);
+	RzBinEndianReader *r = RzBinEndianReader_from_file(bf, ".debug_addr");
 	RET_NULL_IF_FAIL(r);
 	return rz_bin_dwarf_addr_new(r);
 }
