@@ -20,7 +20,7 @@ static bool RzBinDwarfLocation_move(RzBinDwarfLocation *self, RzBinDwarfLocation
 		self->value.location = NULL;
 		break;
 	case RzBinDwarfLocationKind_BYTES:
-		RzBinDwarfBlock_move(&self->bytes, &out->bytes);
+		memcpy(&out->bytes, &self->bytes, sizeof(RzBinDwarfBlock));
 		break;
 	case RzBinDwarfLocationKind_COMPOSITE:
 		self->composite = NULL;
@@ -38,16 +38,15 @@ static bool RzBinDwarfLocation_move(RzBinDwarfLocation *self, RzBinDwarfLocation
 
 static void OperationEvaluationResult_fini(OperationEvaluationResult *self) {
 	rz_bin_dwarf_location_fini(&self->complete);
-	RzBinDwarfEvaluationResult_fini(&self->waiting._2);
 }
 
-RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const RzBinDwarfEncoding *encoding) {
-	rz_return_val_if_fail(self && reader && encoding, false);
+RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *R, const RzBinDwarfEncoding *encoding) {
+	rz_return_val_if_fail(self && R && encoding, false);
 	rz_mem_memzero(self, sizeof(Operation));
 	U8_OR_RET_FALSE(self->opcode);
 	switch (self->opcode) {
 	case DW_OP_addr:
-		RET_FALSE_IF_FAIL(read_address(reader, &self->address.address, encoding->address_size));
+		RET_FALSE_IF_FAIL(R_read_address(R, &self->address.address, encoding->address_size));
 		self->kind = OPERATION_KIND_ADDRESS;
 		break;
 	case DW_OP_deref:
@@ -371,7 +370,7 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 		break;
 	}
 	case DW_OP_call_ref: {
-		RET_FALSE_IF_FAIL(read_address(reader, &self->call.offset, encoding->address_size));
+		RET_FALSE_IF_FAIL(R_read_address(R, &self->call.offset, encoding->address_size));
 		self->kind = OPERATION_KIND_CALL;
 		break;
 	}
@@ -391,7 +390,7 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 	}
 	case DW_OP_implicit_value: {
 		ULE128_OR_RET_FALSE(self->implicit_value.length);
-		RET_FALSE_IF_FAIL(read_block(reader, &self->implicit_value));
+		RET_FALSE_IF_FAIL(R_read_block(R, &self->implicit_value));
 		self->kind = OPERATION_KIND_IMPLICIT_VALUE;
 		break;
 	}
@@ -401,11 +400,11 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 	case DW_OP_implicit_pointer:
 	case DW_OP_GNU_implicit_pointer: {
 		if (encoding->version == 2) {
-			RET_FALSE_IF_FAIL(read_address(
-				reader, &self->implicit_pointer.value, encoding->address_size));
+			RET_FALSE_IF_FAIL(R_read_address(
+				R, &self->implicit_pointer.value, encoding->address_size));
 		} else {
-			RET_FALSE_IF_FAIL(read_offset(
-				reader, &self->implicit_pointer.value, encoding->is_64bit));
+			RET_FALSE_IF_FAIL(R_read_offset(
+				R, &self->implicit_pointer.value, encoding->is_64bit));
 		}
 		SLE128_OR_RET_FALSE(self->implicit_pointer.byte_offset);
 		self->kind = OPERATION_KIND_IMPLICIT_POINTER;
@@ -424,7 +423,7 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 	case DW_OP_entry_value:
 	case DW_OP_GNU_entry_value: {
 		ULE128_OR_RET_FALSE(self->entry_value.expression.length);
-		RET_FALSE_IF_FAIL(read_block(reader, &self->entry_value.expression));
+		RET_FALSE_IF_FAIL(R_read_block(R, &self->entry_value.expression));
 		self->kind = OPERATION_KIND_ENTRY_VALUE;
 		break;
 	}
@@ -432,7 +431,7 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 	case DW_OP_GNU_const_type: {
 		ULE128_OR_RET_FALSE(self->typed_literal.base_type);
 		U8_OR_RET_FALSE(self->typed_literal.value.length);
-		RET_FALSE_IF_FAIL(read_block(reader, &self->typed_literal.value));
+		RET_FALSE_IF_FAIL(R_read_block(R, &self->typed_literal.value));
 		self->kind = OPERATION_KIND_TYPED_LITERAL;
 		break;
 	}
@@ -505,6 +504,8 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 		}
 		break;
 	}
+	case DW_OP_GNU_uninit:
+	case DW_OP_GNU_encoded_addr:
 	case DW_OP_hi_user:
 	default:
 		RZ_LOG_WARN("Unsupported opcode %s 0x%" PFMT32x "\n",
@@ -514,19 +515,6 @@ RZ_IPI bool Operation_parse(Operation *self, RzBinEndianReader *reader, const Rz
 	return true;
 }
 
-static void Operation_fini(Operation *self) {
-	if (!self) {
-		return;
-	}
-	if (self->kind == OPERATION_KIND_IMPLICIT_VALUE) {
-		RzBinDwarfBlock_fini(&self->implicit_value);
-	} else if (self->kind == OPERATION_KIND_ENTRY_VALUE) {
-		RzBinDwarfBlock_fini(&self->entry_value.expression);
-	} else if (self->kind == OPERATION_KIND_TYPED_LITERAL) {
-		RzBinDwarfBlock_fini(&self->typed_literal.value);
-	}
-}
-
 static bool Evaluation_pop(RzBinDwarfEvaluation *self, RzBinDwarfValue *value) {
 	if (rz_vector_len(&self->stack) <= 0) {
 		return false;
@@ -549,8 +537,8 @@ static bool Evaluation_push(RzBinDwarfEvaluation *self, RzBinDwarfValue *value)
 	return status;
 }
 
-static bool compute_pc(RzBinEndianReader *pc, const RzBinEndianReader *bytecode, st16 offset) {
-	return rz_buf_seek(pc->buffer, offset, RZ_BUF_CUR) >= 0;
+static bool compute_pc(RzBinEndianReader *pc, st16 offset) {
+	return R_seek(pc, offset, RZ_BUF_CUR);
 }
 
 static RzBinDwarfValueType ValueType_from_name(const char *name, ut8 byte_size) {
@@ -620,7 +608,7 @@ static RzBinDwarfValueType ValueType_from_die(
 	ut8 byte_size = 0;
 	const char *name = NULL;
 	DW_ATE ate = 0;
-	rz_vector_foreach(&die->attrs, attr) {
+	rz_vector_foreach (&die->attrs, attr) {
 		switch (attr->at) {
 		case DW_AT_name:
 			name = rz_bin_dwarf_attr_string(attr, (RzBinDWARF *)dw, 0);
@@ -663,19 +651,9 @@ static ut64 addrmask_from_size(uint8_t size) {
 				      : (1ULL << (size * 8)) - 1);
 }
 
-static void RzBinDwarfExprStackItem_fini(RzBinDwarfExprStackItem const *self) {
-	if (!self) {
-		return;
-	}
-	RzBinEndianReader_free(self->bytecode);
-	RzBinEndianReader_free(self->pc);
-}
-
 static void RzBinDwarfExprStackItem_cpy(RzBinDwarfExprStackItem *dst, RzBinDwarfExprStackItem *src) {
 	rz_return_if_fail(dst && src);
 	memcpy(dst, src, sizeof(RzBinDwarfExprStackItem));
-	dst->pc = RzBinEndianReader_clone(src->pc);
-	dst->bytecode = RzBinEndianReader_clone(src->bytecode);
 }
 
 RZ_IPI void RzBinDwarfPiece_fini(RzBinDwarfPiece *x) {
@@ -686,7 +664,6 @@ RZ_IPI void RzBinDwarfPiece_fini(RzBinDwarfPiece *x) {
 }
 
 RZ_VECTOR_FINI_T(Value, Value_fini);
-RZ_VECTOR_FINI_T(RzBinDwarfExprStackItem, RzBinDwarfExprStackItem_fini);
 RZ_VECTOR_FINI_T(RzBinDwarfPiece, RzBinDwarfPiece_fini);
 
 RZ_IPI void RzBinDwarfPiece_cpy(RzBinDwarfPiece *dst, RzBinDwarfPiece *src) {
@@ -707,27 +684,25 @@ RZ_API RZ_OWN RzBinDwarfEvaluation *rz_bin_dwarf_evaluation_new(
 	const RzBinDwarfEncoding *encoding = &unit->hdr.encoding;
 	const ut64 addr_mask = addrmask_from_size(encoding->address_size);
 	self->addr_mask = addr_mask;
-	self->bytecode = byte_code;
 	self->encoding = encoding;
-	self->pc = RzBinEndianReader_clone(byte_code);
+	R_clone(byte_code, &self->bytecode);
+	R_clone(byte_code, &self->pc);
 	self->dw = dw;
 	self->unit = unit;
 	self->fn_die = fn_die;
 	rz_vector_init(&self->stack, sizeof(RzBinDwarfValue), RzVector_Value_fini, NULL);
-	rz_vector_init(&self->expression_stack, sizeof(RzBinDwarfExprStackItem), RzVector_RzBinDwarfExprStackItem_fini, NULL);
+	rz_vector_init(&self->expression_stack, sizeof(RzBinDwarfExprStackItem), NULL, NULL);
 	rz_vector_init(&self->result, sizeof(RzBinDwarfPiece), RzVector_RzBinDwarfPiece_fini, NULL);
 	return self;
 }
 
 RZ_API RZ_OWN RzBinDwarfEvaluation *rz_bin_dwarf_evaluation_new_from_block(
-	RZ_BORROW RZ_NONNULL const RzBinDwarfBlock *block,
+	RZ_BORROW RZ_NONNULL RzBinDwarfBlock *block,
 	RZ_BORROW RZ_NONNULL const RzBinDWARF *dw,
 	RZ_BORROW RZ_NULLABLE const RzBinDwarfCompUnit *unit,
 	RZ_BORROW RZ_NULLABLE const RzBinDwarfDie *die) {
 	rz_return_val_if_fail(block && dw, NULL);
-	RzBinEndianReader *r = RzBinDwarfBlock_as_reader(block);
-	RET_NULL_IF_FAIL(r);
-	RzBinDwarfEvaluation *self = rz_bin_dwarf_evaluation_new(r, dw, unit, die);
+	RzBinDwarfEvaluation *self = rz_bin_dwarf_evaluation_new(block, dw, unit, die);
 	RET_NULL_IF_FAIL(self);
 	return self;
 }
@@ -736,8 +711,6 @@ RZ_API void rz_bin_dwarf_evaluation_free(RZ_OWN RzBinDwarfEvaluation *self) {
 	if (!self) {
 		return;
 	}
-	RzBinEndianReader_free(self->pc);
-	RzBinEndianReader_free(self->bytecode);
 	rz_vector_fini(&self->stack);
 	rz_vector_fini(&self->expression_stack);
 	rz_vector_fini(&self->result);
@@ -747,8 +720,8 @@ RZ_API void rz_bin_dwarf_evaluation_free(RZ_OWN RzBinDwarfEvaluation *self) {
 RZ_IPI void rz_bin_dwarf_evaluation_cpy(RzBinDwarfEvaluation *dst, RzBinDwarfEvaluation *src) {
 	rz_return_if_fail(dst && src);
 	memcpy(dst, src, sizeof(RzBinDwarfEvaluation));
-	dst->pc = RzBinEndianReader_clone(src->pc);
-	dst->bytecode = RzBinEndianReader_clone(src->bytecode);
+	dst->pc = src->pc;
+	dst->bytecode = src->bytecode;
 	rz_vector_clone_intof(&dst->stack, &src->stack,
 		(RzVectorItemCpyFunc)Value_cpy);
 	rz_vector_clone_intof(&dst->expression_stack, &src->expression_stack,
@@ -757,27 +730,16 @@ RZ_IPI void rz_bin_dwarf_evaluation_cpy(RzBinDwarfEvaluation *dst, RzBinDwarfEva
 		(RzVectorItemCpyFunc)RzBinDwarfPiece_cpy);
 }
 
-RZ_IPI void RzBinDwarfEvaluationResult_fini(RzBinDwarfEvaluationResult *self) {
-	if (self->kind == EvaluationResult_REQUIRES_ENTRY_VALUE) {
-		RzBinDwarfBlock_fini(&self->requires_entry_value.expression);
-	}
-}
-
 RZ_API void RzBinDwarfEvaluationResult_free(RZ_OWN RzBinDwarfEvaluationResult *self) {
 	if (!self) {
 		return;
 	}
-	RzBinDwarfEvaluationResult_fini(self);
 	free(self);
 }
 
 RZ_IPI void RzBinDwarfEvaluationResult_cpy(RzBinDwarfEvaluationResult *dst, RzBinDwarfEvaluationResult *src) {
 	rz_return_if_fail(dst && src);
 	memcpy(dst, src, sizeof(RzBinDwarfEvaluationResult));
-	if (src->kind == EvaluationResult_REQUIRES_ENTRY_VALUE) {
-		RzBinDwarfBlock_cpy(&dst->requires_entry_value.expression,
-			&src->requires_entry_value.expression);
-	}
 }
 
 #define CHECK_DEFER \
@@ -785,7 +747,7 @@ RZ_IPI void RzBinDwarfEvaluationResult_cpy(RzBinDwarfEvaluationResult *dst, RzBi
 		if (rz_vector_len(&self->stack) >= 1) { \
 			RzBinDwarfValue *val = rz_vector_tail(&self->stack); \
 			if (val->type == RzBinDwarfValueType_LOCATION) { \
-				rz_buf_seek(self->pc->buffer, offset, RZ_BUF_SET); \
+				R_seek(R, offset, RZ_BUF_SET); \
 				out->kind = OperationEvaluationResult_WAITING_RESOLVE; \
 				goto ok; \
 			} \
@@ -797,7 +759,7 @@ RZ_IPI void RzBinDwarfEvaluationResult_cpy(RzBinDwarfEvaluationResult *dst, RzBi
 			RzBinDwarfValue *a = rz_vector_tail(&self->stack); \
 			RzBinDwarfValue *b = rz_vector_index_ptr(&self->stack, rz_vector_len(&self->stack) - 2); \
 			if (a->type == RzBinDwarfValueType_LOCATION || b->type == RzBinDwarfValueType_LOCATION) { \
-				rz_buf_seek(self->pc->buffer, offset, RZ_BUF_SET); \
+				R_seek(R, offset, RZ_BUF_SET); \
 				out->kind = OperationEvaluationResult_WAITING_RESOLVE; \
 				goto ok; \
 			} \
@@ -834,11 +796,11 @@ RZ_IPI void RzBinDwarfEvaluationResult_cpy(RzBinDwarfEvaluationResult *dst, RzBi
 
 static bool Evaluation_evaluate_one_operation(
 	RzBinDwarfEvaluation *self, OperationEvaluationResult *out) {
-	RzBinEndianReader *reader = self->pc;
+	RzBinEndianReader *R = &self->pc;
 	Operation operation = { 0 };
 	bool ret = false;
-	ut64 offset = rz_buf_tell(reader->buffer);
-	OK_OR(Operation_parse(&operation, self->pc, self->encoding),
+	ut64 offset = R_tell(R);
+	OK_OR(Operation_parse(&operation, R, self->encoding),
 		out->kind = OperationEvaluationResult_DECODE_ERROR;
 		goto ok);
 
@@ -949,7 +911,7 @@ static bool Evaluation_evaluate_one_operation(
 		bool status = Evaluation_pop(self, &src1) &&
 			Value_to_u64(&src1, self->addr_mask, &entry) &&
 			(entry != 0) &&
-			compute_pc(self->pc, self->bytecode, operation.bra.target);
+			compute_pc(R, operation.bra.target);
 		Value_fini(&src1);
 		ERR_IF_FAIL(status);
 		break;
@@ -961,7 +923,7 @@ static bool Evaluation_evaluate_one_operation(
 	case OPERATION_KIND_LT: BINARY_OP(Value_lt);
 	case OPERATION_KIND_NE: BINARY_OP(Value_ne);
 	case OPERATION_KIND_SKIP: {
-		ERR_IF_FAIL(compute_pc(self->pc, self->bytecode, operation.skip.target));
+		ERR_IF_FAIL(compute_pc(R, operation.skip.target));
 		break;
 	}
 	case OPERATION_KIND_UNSIGNED_CONSTANT: {
@@ -1061,7 +1023,7 @@ static bool Evaluation_evaluate_one_operation(
 	}
 	case OPERATION_KIND_NOP: break;
 	case OPERATION_KIND_PUSH_OBJECT_ADDRESS: {
-		OK_OR_ERR(self->object_address, RZ_LOG_ERROR("object address not set"));
+		OK_OR_ERR(self->object_address, RZ_LOG_ERROR("object address not set\n"));
 		RzBinDwarfValue v = {
 			.type = RzBinDwarfValueType_GENERIC,
 			.generic = *self->object_address,
@@ -1121,7 +1083,7 @@ static bool Evaluation_evaluate_one_operation(
 	case OPERATION_KIND_IMPLICIT_VALUE: {
 		out->kind = OperationEvaluationResult_COMPLETE;
 		out->complete.kind = RzBinDwarfLocationKind_BYTES;
-		ERR_IF_FAIL(RzBinDwarfBlock_move(&operation.implicit_value, &out->complete.bytes));
+		memcpy(&out->complete.bytes, &operation.implicit_value, sizeof(RzBinDwarfBlock));
 		goto ok;
 	}
 	case OPERATION_KIND_STACK_VALUE: {
@@ -1146,8 +1108,7 @@ static bool Evaluation_evaluate_one_operation(
 	case OPERATION_KIND_ENTRY_VALUE: {
 		out->kind = OperationEvaluationResult_WAITING;
 		out->waiting._1 = EvaluationStateWaiting_ENTRY_VALUE;
-		ERR_IF_FAIL(RzBinDwarfBlock_move(&operation.entry_value.expression,
-			&out->waiting._2.requires_entry_value.expression));
+		memcpy(&out->waiting._2.requires_entry_value.expression, &operation.entry_value.expression, sizeof(RzBinDwarfBlock));
 		goto ok;
 	}
 	case OPERATION_KIND_ADDRESS: {
@@ -1158,7 +1119,7 @@ static bool Evaluation_evaluate_one_operation(
 	}
 	case OPERATION_KIND_ADDRESS_INDEX: {
 		ut64 addr = 0;
-		if (self->dw && self->unit && rz_bin_dwarf_addr_get(self->dw->addr, &addr, self->unit->hdr.encoding.address_size, self->unit->addr_base, operation.address_index.index)) {
+		if (self->dw && self->unit && rz_bin_dwarf_addr(self->dw) && rz_bin_dwarf_addr_get(rz_bin_dwarf_addr(self->dw), &addr, self->unit->hdr.encoding.address_size, self->unit->addr_base, operation.address_index.index)) {
 			out->kind = OperationEvaluationResult_COMPLETE;
 			out->complete.kind = RzBinDwarfLocationKind_ADDRESS;
 			out->complete.address = addr;
@@ -1180,13 +1141,11 @@ static bool Evaluation_evaluate_one_operation(
 
 	case OPERATION_KIND_TYPED_LITERAL: {
 		RzBinDwarfValueType typ = ValueType_from_die(self, self->dw, operation.typed_literal.base_type);
-		RzBinEndianReader *r = RzBinDwarfBlock_as_reader(&operation.typed_literal.value);
-		ERR_IF_FAIL(r);
 		Value dst = { 0 };
-		bool success = Value_parse_into(&dst, typ, reader) &&
-			Evaluation_push(self, &dst);
-		RzBinEndianReader_free(r);
-		OK_OR_ERR(success, Value_fini(&dst));
+		OK_OR_ERR(
+			Value_parse_into(&dst, typ, &operation.typed_literal.value) &&
+				Evaluation_push(self, &dst),
+			Value_fini(&dst));
 		break;
 	}
 	case OPERATION_KIND_CONVERT: {
@@ -1239,12 +1198,11 @@ ok:
 	goto clean;
 err:
 clean:
-	Operation_fini(&operation);
 	return ret;
 }
 
 static bool Evaluation_end_of_expression(RzBinDwarfEvaluation *self) {
-	if (rz_buf_tell(self->pc->buffer) >= rz_buf_size(self->pc->buffer)) {
+	if (R_tell(&self->pc) >= R_size(&self->pc)) {
 		if (rz_vector_empty(&self->expression_stack)) {
 			return true;
 		}
@@ -1323,7 +1281,7 @@ RZ_API bool rz_bin_dwarf_evaluation_evaluate(RZ_BORROW RZ_NONNULL RzBinDwarfEval
 				}
 			} else {
 				Operation operation = { 0 };
-				ERR_IF_FAIL(Operation_parse(&operation, self->pc, self->encoding));
+				ERR_IF_FAIL(Operation_parse(&operation, &self->pc, self->encoding));
 				if (operation.kind != OPERATION_KIND_PIECE) {
 					self->state.kind = EVALUATION_STATE_ERROR;
 					goto err;
@@ -1400,7 +1358,7 @@ RZ_API RZ_BORROW RzVector /*<RzBinDwarfPiece>*/ *rz_bin_dwarf_evaluation_result(
 	if (self->state.kind == EVALUATION_STATE_COMPLETE) {
 		return &self->result;
 	}
-	RZ_LOG_ERROR("Called `Evaluation::result` on an `Evaluation` that has not been completed");
+	RZ_LOG_ERROR("Called `Evaluation::result` on an `Evaluation` that has not been completed\n");
 	return NULL;
 }
 
@@ -1460,14 +1418,15 @@ RZ_API RZ_OWN RzBinDwarfLocation *rz_bin_dwarf_location_from_block(
 	}
 	loc->encoding = unit->hdr.encoding;
 
+	RzBinEndianReader R = { 0 };
+	R_clone(block, &R);
+
 	if (rz_bin_dwarf_block_empty(block)) {
 		loc->kind = RzBinDwarfLocationKind_EMPTY;
-	} else if (!rz_bin_dwarf_block_valid(block)) {
-		loc->kind = RzBinDwarfLocationKind_DECODE_ERROR;
 	} else {
 		RzBinDwarfEvaluationResult *result = RZ_NEW0(RzBinDwarfEvaluationResult);
 		RET_NULL_IF_FAIL(result);
-		eval = rz_bin_dwarf_evaluation_new_from_block(block, dw, unit, die);
+		eval = rz_bin_dwarf_evaluation_new_from_block(&R, dw, unit, die);
 		ERR_IF_FAIL(eval);
 		if (!(rz_bin_dwarf_evaluation_evaluate(eval, result) &&
 			    RzBinDwarfEvaluationResult_to_loc(eval, result, loc))) {
@@ -1604,15 +1563,11 @@ static void Operation_dump(Operation const *op, RzStrBuf *buf) {
 	}
 }
 
-static void vec_Operation_free(void *e, void *u) {
-	Operation_fini(e);
-}
-
 static RzVector /*<Operation>*/ *rz_bin_dwarf_expression_parse(
-	RzBinEndianReader *reader, const RzBinDwarfEncoding *encoding) {
-	RzVector *exprs = rz_vector_new(sizeof(Operation), vec_Operation_free, NULL);
+	RzBinEndianReader *R, const RzBinDwarfEncoding *encoding) {
+	RzVector *exprs = rz_vector_new(sizeof(Operation), NULL, NULL);
 	Operation op = { 0 };
-	while (Operation_parse(&op, reader, encoding)) {
+	while (Operation_parse(&op, R, encoding)) {
 		rz_vector_push(exprs, &op);
 	}
 	return exprs;
@@ -1624,10 +1579,9 @@ RZ_API void rz_bin_dwarf_expression_dump(
 	RZ_BORROW RZ_NONNULL RzStrBuf *sb,
 	RZ_BORROW RZ_NONNULL const RzBinDWARFDumpOption *opt) {
 	rz_return_if_fail(encoding && block && sb && opt);
-	RzBinEndianReader *reader = RzBinDwarfBlock_as_reader(block);
-	OK_OR(reader, return);
-	RzVector *exprs = rz_bin_dwarf_expression_parse(reader, encoding);
-	RzBinEndianReader_free(reader);
+	RzBinEndianReader R = { 0 };
+	R_clone(block, &R);
+	RzVector *exprs = rz_bin_dwarf_expression_parse(&R, encoding);
 	OK_OR(exprs, return);
 
 	rz_strbuf_append(sb, "expressions: [");
@@ -1638,7 +1592,7 @@ RZ_API void rz_bin_dwarf_expression_dump(
 	Operation *op = NULL;
 	ut32 i;
 	const ut32 end = rz_vector_len(exprs) - 1;
-	rz_vector_enumerate(exprs, op, i) {
+	rz_vector_enumerate (exprs, op, i) {
 		rz_strbuf_append(sb, rz_str_get(opt->expr_indent));
 		Operation_dump(op, sb);
 		if (i < end) {
@@ -1685,7 +1639,7 @@ RZ_API void rz_bin_dwarf_loclist_dump(
 	rz_pvector_foreach (&loclist->entries, it) {
 		const RzBinDwarfLocListEntry *entry = *it;
 		rz_strbuf_appendf(sb, "%s(0x%" PFMT64x ", %" PFMT64d "):",
-			rz_str_get(opt->loclist_indent), entry->range->begin, entry->range->end - entry->range->begin);
+			rz_str_get(opt->loclist_indent), entry->range.begin, entry->range.end - entry->range.begin);
 
 		if (entry->location) {
 			rz_strbuf_append(sb, " ");
@@ -1717,7 +1671,7 @@ RZ_API void rz_bin_dwarf_location_composite_dump(
 	ut32 i = 0;
 	const ut32 end = rz_vector_len(composite) - 1;
 	RzBinDwarfPiece *piece = NULL;
-	rz_vector_enumerate(composite, piece, i) {
+	rz_vector_enumerate (composite, piece, i) {
 		rz_strbuf_append(sb, rz_str_get(opt->composite_indent));
 		rz_strbuf_appendf(sb, "(.%" PFMT64u ", %" PFMT64u "): ", piece->bit_offset, piece->size_in_bits);
 
