@@ -115,9 +115,7 @@ RzBinPlugin rz_bin_plugin_cgc = {
 	.maps = &maps,
 	.sections = &sections,
 	.symbols = &symbols,
-	.minstrlen = 4,
 	.imports = &imports,
-	.strings = &strings,
 	.info = &info,
 	.fields = &fields,
 	.size = &size,
