@@ -62,7 +62,7 @@ static bool load_buffer(RzBinFile *bf, RzBinObject *obj, RzBuffer *buf, Sdb *sdb
 	if (!dol) {
 		return false;
 	}
-	char *lowername = strdup(bf->file);
+	char *lowername = rz_str_dup(bf->file);
 	if (!lowername) {
 		goto dol_err;
 	}
@@ -123,7 +123,7 @@ static RzPVector /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 	}
 	/* bss section */
 	s = RZ_NEW0(RzBinSection);
-	s->name = strdup("bss");
+	s->name = rz_str_dup("bss");
 	s->paddr = 0;
 	s->vaddr = dol->bss_addr;
 	s->size = dol->bss_size;
@@ -134,14 +134,14 @@ static RzPVector /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 	return ret;
 }
 
-static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
+static RzPVector /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
 	rz_return_val_if_fail(bf && bf->o && bf->o->bin_obj, NULL);
-	RzList *ret = rz_list_new();
+	RzPVector *ret = rz_pvector_new(NULL);
 	RzBinAddr *addr = RZ_NEW0(RzBinAddr);
 	DolHeader *dol = bf->o->bin_obj;
 	addr->vaddr = (ut64)dol->entrypoint;
 	addr->paddr = addr->vaddr & 0xFFFF;
-	rz_list_append(ret, addr);
+	rz_pvector_push(ret, addr);
 	return ret;
 }
 
@@ -151,12 +151,12 @@ static RzBinInfo *info(RzBinFile *bf) {
 	if (!ret) {
 		return NULL;
 	}
-	ret->file = strdup(bf->file);
+	ret->file = rz_str_dup(bf->file);
 	ret->big_endian = true;
-	ret->type = strdup("ROM");
-	ret->machine = strdup("Nintendo Wii");
-	ret->os = strdup("wii-ios");
-	ret->arch = strdup("ppc");
+	ret->type = rz_str_dup("ROM");
+	ret->machine = rz_str_dup("Nintendo Wii");
+	ret->os = rz_str_dup("wii-ios");
+	ret->arch = rz_str_dup("ppc");
 	ret->has_va = true;
 	ret->bits = 32;
 
