@@ -25,8 +25,6 @@ RzBinPlugin rz_bin_plugin_elf = {
 	.sections = &sections,
 	.symbols = &symbols,
 	.imports = &imports,
-	.strings = &strings,
-	.minstrlen = 4,
 	.info = &info,
 	.fields = &fields,
 	.header = &headers,
