@@ -112,7 +112,7 @@ static RzPVector /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
 	return vec;
 }
 
-static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
+static RzPVector /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
 	if (!bf) {
 		return NULL;
 	}
@@ -121,7 +121,7 @@ static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
 		return NULL;
 	}
 
-	return rz_list_clone(bin_info_obj->entry_list);
+	return rz_pvector_clone(bin_info_obj->entry_vec);
 }
 
 static RzPVector /*<RzBinString *>*/ *strings(RzBinFile *bf) {
@@ -133,12 +133,22 @@ static RzPVector /*<RzBinString *>*/ *strings(RzBinFile *bf) {
 		return NULL;
 	}
 
+	RzPVector *pvec = rz_pvector_new((RzPVectorFree)rz_bin_string_free);
+	if (!pvec || !rz_pvector_reserve(pvec, rz_list_length(bin_info_obj->string_list))) {
+		rz_pvector_free(pvec);
+		return NULL;
+	}
 	RzListIter *iter;
 	RzBinString *bstr;
-	RzPVector *pvec = rz_pvector_new((RzPVectorFree)bin_info_obj->string_list->free);
 	rz_list_foreach (bin_info_obj->string_list, iter, bstr) {
-		rz_pvector_push(pvec, bstr);
+		if (bstr) {
+			rz_pvector_push(pvec, bstr);
+		}
 	}
+	RzListFree free_cb = bin_info_obj->string_list->free;
+	bin_info_obj->string_list->free = NULL;
+	rz_list_purge(bin_info_obj->string_list);
+	bin_info_obj->string_list->free = free_cb;
 	return pvec;
 }
 
