@@ -122,7 +122,7 @@ static char *signature(RzBinFile *bf, bool json) {
 	if (json) {
 		PJ *pj = pj_new();
 		if (!pj) {
-			return strdup("{}");
+			return rz_str_dup("{}");
 		}
 		rz_pkcs7_cms_json(bin->cms, pj);
 		return pj_drain(pj);
@@ -146,7 +146,7 @@ static RzPVector /*<RzBinField *>*/ *fields(RzBinFile *bf) {
 	RzListIter *it;
 	Pe_image_rich_entry *rich;
 	rz_list_foreach (bin->rich_entries, it, rich) {
-		rz_pvector_push(ret, rz_bin_field_new(addr, addr, 0, "RICH_ENTRY_NAME", strdup(rich->productName), "s", false));
+		rz_pvector_push(ret, rz_bin_field_new(addr, addr, 0, "RICH_ENTRY_NAME", rz_str_dup(rich->productName), "s", false));
 		ROWL(tmpbuf, "RICH_ENTRY_ID", 2, rich->productId, "x");
 		addr += 2;
 		ROWL(tmpbuf, "RICH_ENTRY_VERSION", 2, rich->minVersion, "x");
