@@ -89,8 +89,8 @@ static bool load_buffer(RzBinFile *bf, RzBinObject *obj, RzBuffer *b, Sdb *sdb)
 	return false;
 }
 
-static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
-	RzList /*<RzBinAddr *>*/ *ret = rz_list_newf(free);
+static RzPVector /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
+	RzPVector /*<RzBinAddr *>*/ *ret = rz_pvector_new(free);
 	if (!ret) {
 		return NULL;
 	}
@@ -98,7 +98,7 @@ static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
 	if (ptr) {
 		ptr->paddr = N64_ROM_START;
 		ptr->vaddr = baddr(bf);
-		rz_list_append(ret, ptr);
+		rz_pvector_push(ret, ptr);
 	}
 	return ret;
 }
@@ -113,7 +113,7 @@ static RzPVector /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 		rz_pvector_free(ret);
 		return NULL;
 	}
-	text->name = strdup("text");
+	text->name = rz_str_dup("text");
 	text->size = rz_buf_size(bf->buf) - N64_ROM_START;
 	text->vsize = text->size;
 	text->paddr = N64_ROM_START;
@@ -135,10 +135,10 @@ static RzBinInfo *info(RzBinFile *bf) {
 	}
 	memcpy(GameName, n64_header.Name, sizeof(n64_header.Name));
 	ret->file = rz_str_newf("%s (%c)", GameName, n64_header.CountryCode);
-	ret->os = strdup("n64");
-	ret->arch = strdup("mips");
-	ret->machine = strdup("Nintendo 64");
-	ret->type = strdup("ROM");
+	ret->os = rz_str_dup("n64");
+	ret->arch = rz_str_dup("mips");
+	ret->machine = rz_str_dup("Nintendo 64");
+	ret->type = rz_str_dup("ROM");
 	ret->bits = 64;
 	ret->has_va = true;
 	ret->big_endian = true;
