@@ -10,16 +10,6 @@
 
 #define HIGHLIGHT_SZ 1024
 
-static const char *help_msg_greater_sign[] = {
-	"Usage:", "[cmd]>[file]", "redirects console from 'cmd' output to 'file'",
-	"[cmd] > [file]", "", "redirect STDOUT of 'cmd' to 'file'",
-	"[cmd] > $alias", "", "save the output of the command as an alias (see $?)",
-	"[cmd] H> [file]", "", "redirect html output of 'cmd' to 'file'",
-	"[cmd] 2> [file]", "", "redirect STDERR of 'cmd' to 'file'",
-	"[cmd] 2> /dev/null", "", "omit the STDERR output of 'cmd'",
-	NULL
-};
-
 struct rz_core_var {
 	const char *name;
 	const char *description;
@@ -74,7 +64,7 @@ struct rz_core_var core_vars[] = {
 struct rz_core_var help_core_vars[] = {
 	{ "flag", "offset of flag" },
 	{ "${ev}", "get value of eval <config variable <ev>" },
-	{ "$alias", "alias commands (simple macros)" },
+	{ "$alias", "alias commands (simple macros, see $?)" },
 	{ "$e{flag}", "end of <flag> (flag->offset + flag->size)" },
 	{ "$k{kv}", "get value of an sdb query value" },
 	{ "$r{reg}", "get value of named register <reg>" },
@@ -241,7 +231,7 @@ RZ_IPI RzCmdStatus rz_generate_random_number_handler(RzCore *core, int argc, con
 	ut64 high = rz_num_math(core->num, uplimit);
 
 	if (low >= high) {
-		RZ_LOG_ERROR("core : Invalid arguments passed to %s : low-limit shouldn't be more then high-limit", argv[0]);
+		RZ_LOG_ERROR("core : Invalid arguments passed to %s : low-limit shouldn't be more then high-limit\n", argv[0]);
 		return RZ_CMD_STATUS_ERROR;
 	}
 
@@ -262,7 +252,7 @@ RZ_IPI RzCmdStatus rz_print_binary_handler(RzCore *core, int argc, const char **
 RZ_IPI RzCmdStatus rz_base64_encode_handler(RzCore *core, int argc, const char **argv) {
 	char *buf = rz_base64_encode_dyn((ut8 *)argv[1], strlen(argv[1]));
 	if (!buf) {
-		RZ_LOG_ERROR("core: Out of memory!");
+		RZ_LOG_ERROR("Out of memory!\n");
 		return RZ_CMD_STATUS_ERROR;
 	}
 	rz_cons_println(buf);
@@ -273,7 +263,7 @@ RZ_IPI RzCmdStatus rz_base64_encode_handler(RzCore *core, int argc, const char *
 RZ_IPI RzCmdStatus rz_base64_decode_handler(RzCore *core, int argc, const char **argv) {
 	ut8 *buf = rz_base64_decode_dyn(argv[1], -1);
 	if (!buf) {
-		RZ_LOG_ERROR("core: Out of memory!");
+		RZ_LOG_ERROR("Base64 string is invalid\n");
 		return RZ_CMD_STATUS_ERROR;
 	}
 	rz_cons_println((char *)buf);
@@ -289,18 +279,42 @@ RZ_IPI RzCmdStatus rz_check_between_handler(RzCore *core, int argc, const char *
 	return RZ_CMD_STATUS_OK;
 }
 
-RZ_IPI RzCmdStatus rz_print_boundaries_prot_handler(RzCore *core, int argc, const char **argv) {
-	const char *mode = rz_str_trim_head_ro(argv[1]);
-	RzList *list = rz_core_get_boundaries_prot(core, -1, mode, "search");
+RZ_IPI RzCmdStatus rz_print_boundaries_prot_handler(RzCore *core, int argc, const char **argv, RzCmdStateOutput *state) {
+
+	RzList *list = rz_core_get_boundaries_select(core, "search.from", "search.to", "search.in");
 	if (!list) {
-		RZ_LOG_ERROR("core: Failed to get boundaries protection values in RzList");
+		RZ_LOG_ERROR("Failed to get boundaries protection values in RzList\n");
 		return RZ_CMD_STATUS_ERROR;
 	}
+
+	rz_cmd_state_output_array_start(state);
+	rz_cmd_state_output_set_columnsf(state, "xxns", "from", "to", "size", "perms");
 	RzListIter *iter;
 	RzIOMap *map;
 	rz_list_foreach (list, iter, map) {
-		rz_cons_printf("0x%" PFMT64x " 0x%" PFMT64x "\n", map->itv.addr, rz_itv_end(map->itv));
+		ut64 from = map->itv.addr;
+		ut64 to = rz_itv_end(map->itv);
+		ut64 size = map->itv.size;
+		const char *perm = rz_str_rwx_i(map->perm);
+		switch (state->mode) {
+		default:
+			rz_cons_printf("0x%" PFMT64x " 0x%" PFMT64x "\n", from, to);
+			break;
+		case RZ_OUTPUT_MODE_JSON:
+			pj_o(state->d.pj);
+			pj_kn(state->d.pj, "from", from);
+			pj_kn(state->d.pj, "to", to);
+			pj_kn(state->d.pj, "size", size);
+			pj_ks(state->d.pj, "perms", perm);
+			pj_end(state->d.pj);
+			break;
+		case RZ_OUTPUT_MODE_TABLE:
+			rz_table_add_rowf(state->d.t, "xxns", from, to, size, perm);
+			break;
+		}
 	}
+
+	rz_cmd_state_output_array_end(state);
 	rz_list_free(list);
 	return RZ_CMD_STATUS_OK;
 }
@@ -447,7 +461,7 @@ RZ_IPI RzCmdDescDetail *rz_cmd_math_help_vars_details_cb(RzCore *core, int argc,
 	if (!details) {
 		return NULL;
 	}
-	details[0].name = (const char *)strdup("Rizin variables");
+	details[0].name = (const char *)rz_str_dup("Rizin variables");
 	if (!details->name) {
 		goto err;
 	}
@@ -459,18 +473,18 @@ RZ_IPI RzCmdDescDetail *rz_cmd_math_help_vars_details_cb(RzCore *core, int argc,
 	int i;
 	for (i = 0; i < RZ_ARRAY_SIZE(core_vars); i++) {
 		struct rz_core_var *var = &core_vars[i];
-		entries[i].text = (char *)strdup(var->name);
-		entries[i].arg_str = strdup("");
-		entries[i].comment = strdup(var->description);
+		entries[i].text = (char *)rz_str_dup(var->name);
+		entries[i].arg_str = rz_str_dup("");
+		entries[i].comment = rz_str_dup(var->description);
 		if (!entries[i].text || !entries[i].arg_str || !entries[i].comment) {
 			goto err;
 		}
 	}
 	for (int j = 0; j < RZ_ARRAY_SIZE(help_core_vars); j++, i++) {
 		struct rz_core_var *var = &help_core_vars[j];
-		entries[i].text = (char *)strdup(var->name);
-		entries[i].arg_str = strdup("");
-		entries[i].comment = strdup(var->description);
+		entries[i].text = (char *)rz_str_dup(var->name);
+		entries[i].arg_str = rz_str_dup("");
+		entries[i].comment = rz_str_dup(var->description);
 		if (!entries[i].text || !entries[i].arg_str || !entries[i].comment) {
 			goto err;
 		}
