@@ -755,12 +755,6 @@ RZ_IPI RzCmdStatus rz_open_binary_file_handler(RzCore *core, int argc, const cha
 	return RZ_CMD_STATUS_OK;
 }
 
-RZ_IPI RzCmdStatus rz_open_binary_rebase_handler(RzCore *core, int argc, const char **argv) {
-	rz_core_bin_rebase(core, rz_num_math(core->num, argv[1]));
-	rz_core_bin_apply_all_info(core, rz_bin_cur(core->bin));
-	return RZ_CMD_STATUS_OK;
-}
-
 RZ_IPI RzCmdStatus rz_open_binary_reload_handler(RzCore *core, int argc, const char **argv) {
 	// XXX: this will reload the bin using the buffer.
 	// An assumption is made that assumes there is an underlying
@@ -877,38 +871,7 @@ RZ_IPI RzCmdStatus rz_open_malloc_handler(RzCore *core, int argc, const char **a
 		RZ_LOG_ERROR("Invalid length %d.\n", len);
 		return RZ_CMD_STATUS_ERROR;
 	}
-
-	RzCmdStatus res = RZ_CMD_STATUS_ERROR;
-	ut8 *data = RZ_NEWS(ut8, len);
-	if (!data) {
-		return RZ_CMD_STATUS_ERROR;
-	}
-	if (!rz_io_read_at(core->io, core->offset, data, len)) {
-		RZ_LOG_ERROR("Cannot read %d bytes from current offset.\n", len);
-		goto err;
-	}
-
-	char uri[100];
-	rz_strf(uri, "malloc://%d", len);
-	RzCoreFile *cfile = rz_core_file_open(core, uri, RZ_PERM_RWX, 0);
-	if (!cfile) {
-		RZ_LOG_ERROR("Cannot open '%s'.\n", uri);
-		goto err;
-	}
-
-	if (!rz_core_bin_load(core, uri, 0)) {
-		RZ_LOG_ERROR("Cannot load binary info of '%s'.\n", uri);
-		goto err;
-	}
-
-	RzIODesc *desc = rz_io_desc_get(core->io, cfile->fd);
-	rz_warn_if_fail(desc);
-	rz_io_desc_write_at(desc, 0, data, len);
-	res = RZ_CMD_STATUS_OK;
-
-err:
-	free(data);
-	return res;
+	return bool2status(rz_core_file_malloc_copy_chunk(core, len, core->offset));
 }
 
 static RzCmdStatus open_nobin_file(RzCore *core, const char *uri, ut64 addr, int perms) {
@@ -1022,7 +985,7 @@ RZ_IPI RzCmdStatus rz_reopen_debug_file_handler(RzCore *core, int argc, const ch
 
 RZ_IPI RzCmdStatus rz_reopen_debug_rzrun_handler(RzCore *core, int argc, const char **argv) {
 	char *file = rz_file_temp("rz-run");
-	char *s = strdup(argv[1]);
+	char *s = rz_str_dup(argv[1]);
 	rz_config_set(core->config, "dbg.profile", file);
 	rz_str_replace_char(s, ',', '\n');
 	rz_file_dump(file, (const ut8 *)s, strlen(s), 0);
@@ -1053,7 +1016,7 @@ static RzCmdStatus reopen_nobin_headers(RzCore *core, int add_perms) {
 		return RZ_CMD_STATUS_ERROR;
 	}
 	int perms = core->io->desc->perm | add_perms;
-	char *fname = strdup(desc->name);
+	char *fname = rz_str_dup(desc->name);
 	if (!fname) {
 		return RZ_CMD_STATUS_ERROR;
 	}
