@@ -183,6 +183,7 @@ static char *detect_go_package_from_name(const char *string) {
 	 * - `type..` because is just the definition of a function linked to a defined go `typedef`
 	 */
 	if (rz_str_startswith(string, "main.") ||
+		rz_str_startswith(string, "type:") ||
 		rz_str_startswith(string, "type..")) {
 		return NULL;
 	}
@@ -207,6 +208,11 @@ static char *detect_go_package_from_name(const char *string) {
 				end = string + i;
 			}
 			break;
+		} else if (string[i] == ':') {
+			if (!end) {
+				end = string + i;
+			}
+			break;
 		} else if (string[i] == '{') {
 			if (!end) {
 				end = string + i;
@@ -242,6 +248,7 @@ static void add_new_library_from_name(RzCore *core, const char *name) {
 
 	RzBinFile *bf = rz_bin_cur(core->bin);
 	if (!bf || !bf->o) {
+		free(libname);
 		return;
 	}
 
@@ -699,7 +706,7 @@ static bool recover_string_at(GoStrRecover *ctx, ut64 str_addr, ut64 str_size) {
 	char *flag = malloc(str_size + n_prefix + 1);
 	char *raw = malloc(str_size + 1);
 	if (!flag || !raw) {
-		RZ_LOG_ERROR("Cannot allocate buffer to read string.");
+		RZ_LOG_ERROR("Cannot allocate buffer to read string.\n");
 		free(flag);
 		free(raw);
 		return false;
@@ -801,7 +808,7 @@ static bool go_is_sign_match(GoStrRecover *ctx, GoStrInfo *info, GoSignature *si
 }
 
 static ut32 decode_one_opcode_size(GoStrRecover *ctx) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(ctx->core->analysis, &aop, ctx->pc, ctx->bytes, ctx->size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -830,7 +837,7 @@ static bool decode_from_table(RzCore *core, GoStrInfo *info, ut64 pc, const ut8
 }
 
 static bool decode_val_set_size(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -842,7 +849,7 @@ static bool decode_val_set_size(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 }
 
 static bool decode_val_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -854,7 +861,7 @@ static bool decode_val_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 }
 
 static bool decode_val_add_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -866,7 +873,7 @@ static bool decode_val_add_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 }
 
 static bool decode_ptr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -878,7 +885,7 @@ static bool decode_ptr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut
 }
 
 static bool decode_disp_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -1192,7 +1199,7 @@ static ut32 golang_recover_string_arm64(GoStrRecover *ctx) {
 }
 
 static bool decode_ldr_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	ut8 tmp[4];
 	ut64 addr = 0;
 
@@ -1268,7 +1275,7 @@ static ut32 golang_recover_string_arm32(GoStrRecover *ctx) {
 }
 
 static bool decode_lui_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -1652,7 +1659,7 @@ static ut32 golang_recover_string_ppc64(GoStrRecover *ctx) {
 }
 
 static bool decode_auipc_set_addr(RzCore *core, GoStrInfo *info, ut64 pc, const ut8 *buffer, const ut32 size) {
-	RzAnalysisOp aop;
+	RzAnalysisOp aop = { 0 };
 	rz_analysis_op_init(&aop);
 	if (rz_analysis_op(core->analysis, &aop, pc, buffer, size, RZ_ANALYSIS_OP_MASK_BASIC) < 1) {
 		rz_analysis_op_fini(&aop);
@@ -1842,7 +1849,8 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 
 	const char *asm_arch = rz_config_get(core->config, "asm.arch");
 	ut32 asm_bits = rz_config_get_i(core->config, "asm.bits");
-	RzListIter *it, *it2;
+	RzListIter *lit;
+	void **vit;
 	RzAnalysisFunction *func;
 	RzAnalysisBlock *block;
 	GoStrRecoverCb recover_cb = NULL;
@@ -1863,6 +1871,7 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 			recover_cb = &golang_recover_string_x64;
 			break;
 		default:
+			RZ_LOG_WARN("golang: string recovery for %s bits %d is not implemented.\n", asm_arch, asm_bits);
 			break;
 		}
 	} else if (!strcmp(asm_arch, "arm")) {
@@ -1874,6 +1883,7 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 			recover_cb = &golang_recover_string_arm64;
 			break;
 		default:
+			RZ_LOG_WARN("golang: string recovery for %s bits %d is not implemented.\n", asm_arch, asm_bits);
 			break;
 		}
 	} else if (!strcmp(asm_arch, "mips")) {
@@ -1885,6 +1895,7 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 			recover_cb = &golang_recover_string_mips64;
 			break;
 		default:
+			RZ_LOG_WARN("golang: string recovery for %s bits %d is not implemented.\n", asm_arch, asm_bits);
 			break;
 		}
 	} else if (!strcmp(asm_arch, "riscv")) {
@@ -1893,6 +1904,7 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 			recover_cb = &golang_recover_string_riscv64;
 			break;
 		default:
+			RZ_LOG_WARN("golang: string recovery for %s bits %d is not implemented.\n", asm_arch, asm_bits);
 			break;
 		}
 	} else if (!strcmp(asm_arch, "ppc")) {
@@ -1901,6 +1913,7 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 			recover_cb = &golang_recover_string_ppc64;
 			break;
 		default:
+			RZ_LOG_WARN("golang: string recovery for %s bits %d is not implemented.\n", asm_arch, asm_bits);
 			break;
 		}
 	} else if (!strcmp(asm_arch, "sysz")) {
@@ -1916,11 +1929,12 @@ RZ_API void rz_core_analysis_resolve_golang_strings(RzCore *core) {
 		return;
 	}
 
-	rz_list_foreach (core->analysis->fcns, it, func) {
+	rz_list_foreach (core->analysis->fcns, lit, func) {
 		if (rz_cons_is_breaked()) {
 			break;
 		}
-		rz_list_foreach (func->bbs, it2, block) {
+		rz_pvector_foreach (func->bbs, vit) {
+			block = (RzAnalysisBlock *)*vit;
 			bytes = malloc(block->size);
 			if (!bytes) {
 				RZ_LOG_ERROR("Failed allocate basic block bytes buffer\n");
