@@ -33,12 +33,12 @@ RZ_API void rz_serialize_core_save(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core,
 	if (snprintf(buf, sizeof(buf), "0x%" PFMT64x, core->offset) < 0) {
 		return;
 	}
-	sdb_set(db, "offset", buf, 0);
+	sdb_set(db, "offset", buf);
 
 	if (snprintf(buf, sizeof(buf), "0x%" PFMT32x, core->blocksize) < 0) {
 		return;
 	}
-	sdb_set(db, "blocksize", buf, 0);
+	sdb_set(db, "blocksize", buf);
 }
 
 static const char *const config_exclude[] = {
@@ -83,14 +83,14 @@ RZ_API bool rz_serialize_core_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core,
 	SUB("debug", rz_serialize_debug_load(subdb, core->dbg, res));
 	SUB("seek", rz_serialize_core_seek_load(subdb, core, res));
 
-	const char *str = sdb_const_get(db, "offset", 0);
+	const char *str = sdb_const_get(db, "offset");
 	if (!str || !*str) {
 		RZ_SERIALIZE_ERR(res, "missing offset in core");
 		return false;
 	}
 	core->offset = strtoull(str, NULL, 0);
 
-	str = sdb_const_get(db, "blocksize", 0);
+	str = sdb_const_get(db, "blocksize");
 	if (!str || !*str) {
 		RZ_SERIALIZE_ERR(res, "missing blocksize in core");
 		return false;
@@ -173,12 +173,12 @@ static void file_save(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core, RZ_NULLABLE c
 	if (!filename) {
 		return;
 	}
-	sdb_set(db, "raw", filename, 0);
+	sdb_set(db, "raw", filename);
 	char *abs = rz_file_abspath(filename);
 	if (!abs) {
 		return;
 	}
-	sdb_set(db, "absolute", abs, 0);
+	sdb_set(db, "absolute", abs);
 	if (prj_file) {
 		char *prj_dir = prj_dir_abs(prj_file);
 		if (!prj_dir) {
@@ -186,7 +186,7 @@ static void file_save(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core, RZ_NULLABLE c
 		}
 		char *rel = prj_relative_make(prj_dir, abs);
 		if (rel) {
-			sdb_set(db, "relative", rel, 0);
+			sdb_set(db, "relative", rel);
 			free(rel);
 		}
 		free(prj_dir);
@@ -224,7 +224,7 @@ static bool file_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core, RZ_NULLABLE c
 	rz_bin_file_delete_all(core->bin);
 
 	FileRet r = FILE_DOES_NOT_EXIST;
-	const char *rel = sdb_const_get(db, "relative", 0);
+	const char *rel = sdb_const_get(db, "relative");
 	if (rel && prj_file) {
 		char *prj_dir = prj_dir_abs(prj_file);
 		if (prj_dir) {
@@ -240,7 +240,7 @@ static bool file_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core, RZ_NULLABLE c
 		return r == FILE_SUCCESS;
 	}
 
-	const char *file = sdb_const_get(db, "absolute", 0);
+	const char *file = sdb_const_get(db, "absolute");
 	if (file) {
 		r = try_load_file(core, file, res);
 	}
@@ -248,7 +248,7 @@ static bool file_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *core, RZ_NULLABLE c
 		return r == FILE_SUCCESS;
 	}
 
-	file = sdb_const_get(db, "raw", 0);
+	file = sdb_const_get(db, "raw");
 	if (file) {
 		r = try_load_file(core, file, res);
 	}
@@ -288,7 +288,7 @@ RZ_API void rz_serialize_core_seek_save(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *c
 		pj_end(j);
 
 		char key[12];
-		sdb_set(db, rz_strf(key, "%" PFMT32d, undo->idx), pj_string(j), 0);
+		sdb_set(db, rz_strf(key, "%" PFMT32d, undo->idx), pj_string(j));
 		pj_free(j);
 	}
 
@@ -334,7 +334,7 @@ typedef struct {
  **/
 static bool seek_load_item(SeekLoadCtx *ctx, const char *k, const char *v) {
 	bool ret = false;
-	char *json_str = strdup(v);
+	char *json_str = rz_str_dup(v);
 	if (!json_str) {
 		return true;
 	}
@@ -372,11 +372,11 @@ static bool seek_load_item(SeekLoadCtx *ctx, const char *k, const char *v) {
 		// Switch to the vector of redos
 		ctx->vec = &ctx->core->seek_history.redos;
 		// Remember we've found the current seek
-		ctx->current_key = strdup(k);
+		ctx->current_key = rz_str_dup(k);
 	} else {
 		if (seek_item.is_current) {
 			// Warn about this additional "current" seek
-			RZ_LOG_WARN("core: Seek history item \"%s\" marked as current, but current already found at \"%s\"!", k, ctx->current_key);
+			RZ_LOG_WARN("Seek history item \"%s\" marked as current, but current already found at \"%s\"!\n", k, ctx->current_key);
 		}
 		rz_vector_push(ctx->vec, &seek_item);
 	}
@@ -388,12 +388,12 @@ out_free_str:
 	return ret;
 }
 
-static int __cmp_num_asc(const void *a, const void *b) {
+static int __cmp_num_asc(const void *a, const void *b, RZ_UNUSED void *user) {
 	const SdbKv *ka = a, *kb = b;
 	// Parse as signed ints but don't bother witb error detection, it'll sort bad and that's it
 	long ia = strtol(sdbkv_key(ka), NULL, 10);
 	long ib = strtol(sdbkv_key(kb), NULL, 10);
-	return ia > ib;
+	return RZ_NUM_CMP(ia, ib);
 }
 
 /**
@@ -414,19 +414,17 @@ RZ_API bool rz_serialize_core_seek_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *c
 	}
 
 	// Sort by (numeric) key
-	SdbList *db_list = sdb_foreach_list(db, false);
-	if (!db_list) {
+	RzPVector *db_items = sdb_get_items(db, false);
+	if (!db_items) {
 		ret = false;
 		goto out_free_parser;
 	}
-	ls_sort(db_list, __cmp_num_asc);
+	rz_pvector_sort(db_items, __cmp_num_asc, NULL);
 
 	// Clear the current history
 	rz_core_seek_reset(core);
 	core->seek_history.saved_set = false;
 
-	SdbKv *kv;
-	SdbListIter *it;
 	SeekLoadCtx ctx = {
 		.core = core,
 		.parser = seek_parser,
@@ -434,7 +432,9 @@ RZ_API bool rz_serialize_core_seek_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *c
 		.vec = &core->seek_history.undos,
 	};
 	bool parsed = true;
-	ls_foreach (db_list, it, kv) {
+	void **it;
+	rz_pvector_foreach (db_items, it) {
+		SdbKv *kv = *it;
 		parsed &= seek_load_item(&ctx, sdbkv_key(kv), sdbkv_value(kv));
 	}
 	ret &= parsed;
@@ -465,13 +465,13 @@ RZ_API bool rz_serialize_core_seek_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzCore *c
 	}
 	ut64 histsize = rz_config_get_i(core->config, "cfg.seek.histsize");
 	if (histsize != 0 && histsize < ulen + rlen) {
-		RZ_LOG_WARN("core: Loaded project seek history exceeds cfg.seek.histsize, increasing that limit.");
+		RZ_LOG_WARN("Loaded project seek history exceeds cfg.seek.histsize, increasing that limit.\n");
 		rz_config_set_i(core->config, "cfg.seek.histsize", ulen + rlen);
 	}
 
 out_free_list:
 	free(ctx.current_key);
-	ls_free(db_list);
+	rz_pvector_free(db_items);
 out_free_parser:
 	rz_key_parser_free(seek_parser);
 	return ret;
