@@ -160,7 +160,7 @@ static bool aes_ecb_fini(RzCrypto *cry) {
 
 RzCryptoPlugin rz_crypto_plugin_aes = {
 	.name = "aes-ecb",
-	.author = "Nettle project (algorithm implementation), pancake (plugin)",
+	.author = "Nettle project,pancake",
 	.license = "LGPL3",
 	.set_key = aes_set_key,
 	.get_key_size = aes_get_key_size,
