@@ -6,6 +6,7 @@
 #include <rz_list.h>
 #include <rz_skiplist.h>
 #include <rz_util/rz_serialize.h>
+#include <rz_util/ht_sp.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -45,12 +46,11 @@ typedef struct rz_flag_item_t {
 
 typedef struct rz_flag_t {
 	RzSpaces spaces; /* handle flag spaces */
-	st64 base; /* base address for all flag items */
 	bool realnames;
 	Sdb *tags;
 	RzNum *num;
 	RzSkipList *by_off; /* flags sorted by offset, value=RzFlagsAtOffset */
-	HtPP *ht_name; /* hashmap key=item name, value=RzFlagItem * */
+	HtSP *ht_name; /* hashmap key=item name, value=RzFlagItem * */
 	RzList /*<RzFlagZoneItem *>*/ *zones;
 } RzFlag;
 
