@@ -27,14 +27,16 @@ KK = 0 - (sum of all bytes)
 
 */
 
-#include "rz_io.h"
-#include "rz_lib.h"
-#include "rz_util.h"
-#include <limits.h> //for INT_MAX
+#include <rz_io.h>
+#include <rz_lib.h>
+#include <rz_util.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
+#include "rz_io_plugins.h"
+
 // struct Rihex : holds sparse buffer + its own fd, for internal management
 typedef struct {
 	int fd;
@@ -43,7 +45,7 @@ typedef struct {
 
 static bool ihex_write(RzIODesc *desc, Rihex *rih);
 
-static int __write(RzIO *io, RzIODesc *fd, const ut8 *buf, int count) {
+static int __write(RzIO *io, RzIODesc *fd, const ut8 *buf, size_t count) {
 	if (!fd || !fd->data || (fd->perm & RZ_PERM_W) == 0 || count <= 0) {
 		return -1;
 	}
@@ -121,7 +123,7 @@ static int fwblock(FILE *fd, ut8 *b, ut32 start_addr, ut32 size) {
 	return 0;
 }
 
-static int __read(RzIO *io, RzIODesc *fd, ut8 *buf, int count) {
+static int __read(RzIO *io, RzIODesc *fd, ut8 *buf, size_t count) {
 	if (!fd || !fd->data || (count <= 0)) {
 		return -1;
 	}
