@@ -3,44 +3,87 @@
 // SPDX-License-Identifier: LGPL-3.0-only
 
 #include <stdio.h>
-#include "rz_util.h"
+#include <rz_util.h>
 
+/**
+ * \brief returns the prev RzList iterator in the list
+ *
+ **/
+RZ_API RZ_BORROW RzListIter *rz_list_iter_get_prev(RZ_NONNULL RzListIter *iter) {
+	rz_return_val_if_fail(iter, NULL);
+	return iter->prev;
+}
 /**
  * \brief returns the next RzList iterator in the list
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_iter_get_next(RzListIter *iter) {
+RZ_API RZ_BORROW RzListIter *rz_list_iter_get_next(RZ_NONNULL RzListIter *iter) {
 	rz_return_val_if_fail(iter, NULL);
-	return iter->n;
+	return iter->next;
+}
+
+/**
+ * \brief returns the value stored in the prev RzList iterator
+ *
+ **/
+RZ_API RZ_BORROW void *rz_list_iter_get_prev_data(RZ_NONNULL RzListIter *iter) {
+	rz_return_val_if_fail(iter, NULL);
+	RzListIter *p = iter->prev;
+	if (!p) {
+		return NULL;
+	}
+	return p->elem;
 }
 
 /**
  * \brief returns the value stored in the next RzList iterator
  *
  **/
-RZ_API RZ_BORROW void *rz_list_iter_get_next_data(RzListIter *iter) {
+RZ_API RZ_BORROW void *rz_list_iter_get_next_data(RZ_NONNULL RzListIter *iter) {
 	rz_return_val_if_fail(iter, NULL);
-	RzListIter *n = iter->n;
+	RzListIter *n = iter->next;
 	if (!n) {
 		return NULL;
 	}
-	return n->data;
+	return n->elem;
 }
 
 /**
- * \brief returns the value stored in the list element
+ * \brief returns the value stored in the list iterator
  *
  **/
-RZ_API void *rz_list_iter_get_data(RzListIter *iter) {
+RZ_API void *rz_list_iter_get_data(RZ_NONNULL RzListIter *iter) {
 	rz_return_val_if_fail(iter, NULL);
-	return iter->data;
+	return iter->elem;
+}
+
+/**
+ * \brief Sets the value stored in the list iterator and returns true if succeeds
+ *
+ **/
+RZ_API bool rz_list_iter_set_data(RZ_NONNULL RzListIter *iter, RZ_NULLABLE void *data) {
+	rz_return_val_if_fail(iter, false);
+	iter->elem = data;
+	return true;
+}
+
+/**
+ * \brief swaps the data held by two iterators and returns true if succeeds
+ *
+ **/
+RZ_API bool rz_list_iter_swap_data(RZ_NONNULL RzListIter *iter0, RZ_NONNULL RzListIter *iter1) {
+	rz_return_val_if_fail(iter0 && iter1, false);
+	void *tmp = iter0->elem;
+	iter0->elem = iter1->elem;
+	iter1->elem = tmp;
+	return true;
 }
 
 /**
  * \brief returns the first RzList iterator int the list
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_iterator(const RzList *list) {
+RZ_API RZ_BORROW RzListIter *rz_list_iterator(RZ_NONNULL const RzList *list) {
 	rz_return_val_if_fail(list, NULL);
 	return list->head;
 }
@@ -53,22 +96,13 @@ RZ_API RZ_BORROW RzListIter *rz_list_push(RZ_NONNULL RzList *list, void *item) {
 	return rz_list_append(list, item);
 }
 
-/**
- * \brief Returns the next element of the list
- *
- **/
-RZ_API RzListIter *rz_list_get_next(RzListIter *iter) {
-	rz_return_val_if_fail(iter, NULL);
-	return iter->n;
-}
-
 /**
  * \brief Returns the first element of the list
  *
  **/
 RZ_API RZ_BORROW void *rz_list_first(RZ_NONNULL const RzList *list) {
 	rz_return_val_if_fail(list, NULL);
-	return list->head ? list->head->data : NULL;
+	return list->head ? list->head->elem : NULL;
 }
 
 /**
@@ -77,7 +111,7 @@ RZ_API RZ_BORROW void *rz_list_first(RZ_NONNULL const RzList *list) {
  **/
 RZ_API RZ_BORROW void *rz_list_last(RZ_NONNULL const RzList *list) {
 	rz_return_val_if_fail(list, NULL);
-	return list->tail ? list->tail->data : NULL;
+	return list->tail ? list->tail->elem : NULL;
 }
 
 /**
@@ -114,7 +148,7 @@ RZ_API void rz_list_purge(RZ_NONNULL RzList *list) {
 
 	RzListIter *it = list->head;
 	while (it) {
-		RzListIter *next = it->n;
+		RzListIter *next = it->next;
 		rz_list_delete(list, it);
 		it = next;
 	}
@@ -126,11 +160,12 @@ RZ_API void rz_list_purge(RZ_NONNULL RzList *list) {
  * \brief Empties the list and frees the list pointer
  *
  **/
-RZ_API void rz_list_free(RzList *list) {
-	if (list) {
-		rz_list_purge(list);
-		free(list);
+RZ_API void rz_list_free(RZ_NULLABLE RzList *list) {
+	if (!list) {
+		return;
 	}
+	rz_list_purge(list);
+	free(list);
 }
 
 /**
@@ -154,10 +189,10 @@ RZ_API bool rz_list_delete_data(RZ_NONNULL RzList *list, void *ptr) {
 RZ_API void rz_list_delete(RZ_NONNULL RzList *list, RZ_NONNULL RzListIter *iter) {
 	rz_return_if_fail(list && iter);
 	rz_list_split_iter(list, iter);
-	if (list->free && iter->data) {
-		list->free(iter->data);
+	if (list->free && iter->elem) {
+		list->free(iter->elem);
 	}
-	iter->data = NULL;
+	iter->elem = NULL;
 	free(iter);
 }
 
@@ -166,13 +201,13 @@ RZ_API void rz_list_split(RZ_NONNULL RzList *list, void *ptr) {
 
 	RzListIter *iter = rz_list_iterator(list);
 	while (iter) {
-		void *item = iter->data;
+		void *item = iter->elem;
 		if (ptr == item) {
 			rz_list_split_iter(list, iter);
 			free(iter);
 			break;
 		}
-		iter = iter->n;
+		iter = iter->next;
 	}
 }
 
@@ -180,16 +215,16 @@ RZ_API void rz_list_split_iter(RZ_NONNULL RzList *list, RZ_NONNULL RzListIter *i
 	rz_return_if_fail(list);
 
 	if (list->head == iter) {
-		list->head = iter->n;
+		list->head = iter->next;
 	}
 	if (list->tail == iter) {
-		list->tail = iter->p;
+		list->tail = iter->prev;
 	}
-	if (iter->p) {
-		iter->p->n = iter->n;
+	if (iter->prev) {
+		iter->prev->next = iter->next;
 	}
-	if (iter->n) {
-		iter->n->p = iter->p;
+	if (iter->next) {
+		iter->next->prev = iter->prev;
 	}
 	list->length--;
 }
@@ -208,10 +243,10 @@ RZ_API bool rz_list_join(RZ_NONNULL RzList *list1, RZ_NONNULL RzList *list2) {
 		list1->head = list2->head;
 		list1->tail = list2->tail;
 	} else {
-		list1->tail->n = list2->head;
-		list2->head->p = list1->tail;
+		list1->tail->next = list2->head;
+		list2->head->prev = list1->tail;
 		list1->tail = list2->tail;
-		list1->tail->n = NULL;
+		list1->tail->next = NULL;
 		list1->sorted = false;
 	}
 	list1->length += list2->length;
@@ -237,7 +272,7 @@ RZ_API RZ_OWN RzList *rz_list_new(void) {
  * \brief Returns a new initialized RzList pointer and sets the free method
  *
  **/
-RZ_API RZ_OWN RzList *rz_list_newf(RzListFree f) {
+RZ_API RZ_OWN RzList *rz_list_newf(RZ_NULLABLE RzListFree f) {
 	RzList *l = rz_list_new();
 	if (l) {
 		l->free = f;
@@ -249,11 +284,14 @@ RZ_API RZ_OWN RzList *rz_list_newf(RzListFree f) {
  * \brief Allocates a new RzList and adds an array elements to it
  *
  **/
-RZ_API RZ_OWN RzList *rz_list_new_from_array(RZ_NONNULL const void **arr, size_t arr_size) {
+RZ_API RZ_OWN RzList *rz_list_new_from_array(const void **arr, size_t arr_size) {
 	RzList *l = rz_list_new();
 	if (!l) {
 		return NULL;
 	}
+	if (!arr) {
+		return l;
+	}
 	size_t i;
 	for (i = 0; i < arr_size; i++) {
 		rz_list_append(l, (void *)arr[i]);
@@ -261,14 +299,33 @@ RZ_API RZ_OWN RzList *rz_list_new_from_array(RZ_NONNULL const void **arr, size_t
 	return l;
 }
 
+/**
+ * \brief Allocates a new RzList and adds all elements of the iterator \p iter to it.
+ * \p iter keeps the ownership over the values.
+ *
+ * \return The produced list. Or NULL in case of failure.
+ **/
+RZ_API RZ_OWN RzList *rz_list_new_from_iterator(RZ_BORROW RZ_NONNULL RzIterator *iter) {
+	rz_return_val_if_fail(iter, NULL);
+	RzList *l = rz_list_new();
+	if (!l) {
+		return NULL;
+	}
+	void **val;
+	rz_iterator_foreach(iter, val) {
+		rz_list_append(l, (void *)*val);
+	}
+	return l;
+}
+
 /**
  * \brief Creates a RzListIter element that can be inserted into a RzList
  *
  **/
-RZ_API RZ_OWN RzListIter *rz_list_item_new(void *data) {
+RZ_API RZ_OWN RzListIter *rz_list_item_new(RZ_NULLABLE void *data) {
 	RzListIter *item = RZ_NEW0(RzListIter);
 	if (item) {
-		item->data = data;
+		item->elem = data;
 	}
 	return item;
 }
@@ -277,7 +334,7 @@ RZ_API RZ_OWN RzListIter *rz_list_item_new(void *data) {
  * \brief Appends at the end of the list a new element
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_append(RZ_NONNULL RzList *list, void *data) {
+RZ_API RZ_BORROW RzListIter *rz_list_append(RZ_NONNULL RzList *list, RZ_NONNULL void *data) {
 	RzListIter *item = NULL;
 
 	rz_return_val_if_fail(list, NULL);
@@ -287,11 +344,11 @@ RZ_API RZ_BORROW RzListIter *rz_list_append(RZ_NONNULL RzList *list, void *data)
 		return item;
 	}
 	if (list->tail) {
-		list->tail->n = item;
+		list->tail->next = item;
 	}
-	item->data = data;
-	item->p = list->tail;
-	item->n = NULL;
+	item->elem = data;
+	item->prev = list->tail;
+	item->next = NULL;
 	list->tail = item;
 	if (!list->head) {
 		list->head = item;
@@ -305,7 +362,7 @@ RZ_API RZ_BORROW RzListIter *rz_list_append(RZ_NONNULL RzList *list, void *data)
  * \brief Appends at the beginning of the list a new element
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_prepend(RZ_NONNULL RzList *list, void *data) {
+RZ_API RZ_BORROW RzListIter *rz_list_prepend(RZ_NONNULL RzList *list, RZ_NONNULL void *data) {
 	rz_return_val_if_fail(list, NULL);
 
 	RzListIter *item = RZ_NEW0(RzListIter);
@@ -313,11 +370,11 @@ RZ_API RZ_BORROW RzListIter *rz_list_prepend(RZ_NONNULL RzList *list, void *data
 		return NULL;
 	}
 	if (list->head) {
-		list->head->p = item;
+		list->head->prev = item;
 	}
-	item->data = data;
-	item->n = list->head;
-	item->p = NULL;
+	item->elem = data;
+	item->next = list->head;
+	item->prev = NULL;
 	list->head = item;
 	if (!list->tail) {
 		list->tail = item;
@@ -331,7 +388,7 @@ RZ_API RZ_BORROW RzListIter *rz_list_prepend(RZ_NONNULL RzList *list, void *data
  * \brief Inserts a new element at the N-th position
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_insert(RZ_NONNULL RzList *list, ut32 n, void *data) {
+RZ_API RZ_BORROW RzListIter *rz_list_insert(RZ_NONNULL RzList *list, ut32 n, RZ_NONNULL void *data) {
 	RzListIter *it, *item;
 	ut32 i;
 
@@ -340,19 +397,19 @@ RZ_API RZ_BORROW RzListIter *rz_list_insert(RZ_NONNULL RzList *list, ut32 n, voi
 	if (!list->head || !n) {
 		return rz_list_prepend(list, data);
 	}
-	for (it = list->head, i = 0; it && it->data; it = it->n, i++) {
+	for (it = list->head, i = 0; it && it->elem; it = it->next, i++) {
 		if (i == n) {
 			item = RZ_NEW(RzListIter);
 			if (!item) {
 				return NULL;
 			}
-			item->data = data;
-			item->n = it;
-			item->p = it->p;
-			if (it->p) {
-				it->p->n = item;
+			item->elem = data;
+			item->next = it;
+			item->prev = it->prev;
+			if (it->prev) {
+				it->prev->next = item;
 			}
-			it->p = item;
+			it->prev = item;
 			list->length++;
 			list->sorted = true;
 			return item;
@@ -376,10 +433,10 @@ RZ_API RZ_OWN void *rz_list_pop(RZ_NONNULL RzList *list) {
 		if (list->head == list->tail) {
 			list->head = list->tail = NULL;
 		} else {
-			list->tail = iter->p;
-			list->tail->n = NULL;
+			list->tail = iter->prev;
+			list->tail->next = NULL;
 		}
-		data = iter->data;
+		data = iter->elem;
 		free(iter);
 		list->length--;
 	}
@@ -400,10 +457,10 @@ RZ_API RZ_OWN void *rz_list_pop_head(RZ_NONNULL RzList *list) {
 		if (list->head == list->tail) {
 			list->head = list->tail = NULL;
 		} else {
-			list->head = iter->n;
-			list->head->p = NULL;
+			list->head = iter->next;
+			list->head->prev = NULL;
 		}
-		data = iter->data;
+		data = iter->elem;
 		free(iter);
 		list->length--;
 	}
@@ -420,48 +477,27 @@ RZ_API ut32 rz_list_del_n(RZ_NONNULL RzList *list, ut32 n) {
 
 	rz_return_val_if_fail(list, false);
 
-	for (it = list->head, i = 0; it && it->data; it = it->n, i++) {
+	for (it = list->head, i = 0; it && it->elem; it = it->next, i++) {
 		if (i == n) {
-			if (!it->p && !it->n) {
+			if (!it->prev && !it->next) {
 				list->head = list->tail = NULL;
-			} else if (!it->p) {
-				it->n->p = NULL;
-				list->head = it->n;
-			} else if (!it->n) {
-				it->p->n = NULL;
-				list->tail = it->p;
+			} else if (!it->prev) {
+				it->next->prev = NULL;
+				list->head = it->next;
+			} else if (!it->next) {
+				it->prev->next = NULL;
+				list->tail = it->prev;
 			} else {
-				it->p->n = it->n;
-				it->n->p = it->p;
+				it->prev->next = it->next;
+				it->next->prev = it->prev;
 			}
-			free(it);
-			list->length--;
+			rz_list_delete(list, it);
 			return true;
 		}
 	}
 	return false;
 }
 
-/**
- * \brief Returns the last element of the list
- *
- **/
-RZ_API RZ_BORROW void *rz_list_get_top(RZ_NONNULL const RzList *list) {
-	rz_return_val_if_fail(list, NULL);
-
-	return list->tail ? list->tail->data : NULL;
-}
-
-/**
- * \brief Returns the first element of the list
- *
- **/
-RZ_API RZ_BORROW void *rz_list_get_bottom(RZ_NONNULL const RzList *list) {
-	rz_return_val_if_fail(list, NULL);
-
-	return list->head ? list->head->data : NULL;
-}
-
 /**
  * \brief Reverses the list
  *
@@ -471,10 +507,10 @@ RZ_API void rz_list_reverse(RZ_NONNULL RzList *list) {
 
 	rz_return_if_fail(list);
 
-	for (it = list->head; it && it->data; it = it->p) {
-		tmp = it->p;
-		it->p = it->n;
-		it->n = tmp;
+	for (it = list->head; it && it->elem; it = it->prev) {
+		tmp = it->prev;
+		it->prev = it->next;
+		it->next = tmp;
 	}
 	tmp = list->head;
 	list->head = list->tail;
@@ -507,27 +543,25 @@ RZ_API RZ_OWN RzList *rz_list_clone(RZ_NONNULL const RzList *list) {
  * \brief Adds an element to a sorted list via the RzListComparator
  *
  **/
-RZ_API RZ_BORROW RzListIter *rz_list_add_sorted(RZ_NONNULL RzList *list, void *data, RZ_NONNULL RzListComparator cmp) {
-	RzListIter *it, *item = NULL;
-
+RZ_API RZ_BORROW RzListIter *rz_list_add_sorted(RZ_NONNULL RzList *list, RZ_NONNULL void *data, RZ_NONNULL RzListComparator cmp, void *user) {
 	rz_return_val_if_fail(list && data && cmp, NULL);
 
-	for (it = list->head; it && it->data && cmp(data, it->data) > 0; it = it->n) {
-		;
+	RzListIter *it, *item = NULL;
+	for (it = list->head; it && it->elem && cmp(data, it->elem, user) > 0; it = it->next) {
 	}
 	if (it) {
 		item = RZ_NEW0(RzListIter);
 		if (!item) {
 			return NULL;
 		}
-		item->n = it;
-		item->p = it->p;
-		item->data = data;
-		item->n->p = item;
-		if (!item->p) {
+		item->next = it;
+		item->prev = it->prev;
+		item->elem = data;
+		item->next->prev = item;
+		if (!item->prev) {
 			list->head = item;
 		} else {
-			item->p->n = item;
+			item->prev->next = item;
 		}
 		list->length++;
 	} else {
@@ -541,17 +575,17 @@ RZ_API RZ_BORROW RzListIter *rz_list_add_sorted(RZ_NONNULL RzList *list, void *d
  * \brief Sets the N-th element of the list
  *
  **/
-RZ_API ut32 rz_list_set_n(RZ_NONNULL RzList *list, ut32 n, void *p) {
+RZ_API ut32 rz_list_set_n(RZ_NONNULL RzList *list, ut32 n, RZ_NONNULL void *data) {
 	RzListIter *it;
 	ut32 i;
 
 	rz_return_val_if_fail(list, false);
-	for (it = list->head, i = 0; it; it = it->n, i++) {
+	for (it = list->head, i = 0; it; it = it->next, i++) {
 		if (i == n) {
 			if (list->free) {
-				list->free(it->data);
+				list->free(it->elem);
 			}
-			it->data = p;
+			it->elem = data;
 			list->sorted = false;
 			return true;
 		}
@@ -568,10 +602,13 @@ RZ_API RZ_BORROW void *rz_list_get_n(RZ_NONNULL const RzList *list, ut32 n) {
 	ut32 i;
 
 	rz_return_val_if_fail(list, NULL);
+	if (n >= list->length) {
+		return NULL;
+	}
 
-	for (it = list->head, i = 0; it && it->data; it = it->n, i++) {
+	for (it = list->head, i = 0; it && it->elem; it = it->next, i++) {
 		if (i == n) {
-			return it->data;
+			return it->elem;
 		}
 	}
 	return NULL;
@@ -609,48 +646,47 @@ RZ_API RZ_BORROW RzListIter *rz_list_find_ptr(RZ_NONNULL const RzList *list, RZ_
  *
  * \return the first RzListIter that is equall to p w.r.t. cmp.
  */
-RZ_API RZ_BORROW RzListIter *rz_list_find(RZ_NONNULL const RzList *list, const void *p, RZ_NONNULL RzListComparator cmp) {
+RZ_API RZ_BORROW RzListIter *rz_list_find(RZ_NONNULL const RzList *list, const void *p, RZ_NONNULL RzListComparator cmp, void *user) {
+	rz_return_val_if_fail(list && cmp, NULL);
+
 	void *q;
 	RzListIter *iter;
-
-	rz_return_val_if_fail(list, NULL);
-
 	rz_list_foreach (list, iter, q) {
-		if (!cmp(p, q)) {
+		if (!cmp(p, q, user)) {
 			return iter;
 		}
 	}
 	return NULL;
 }
 
-static RzListIter *_merge(RzListIter *first, RzListIter *second, RzListComparator cmp) {
+static RzListIter *_merge(RzListIter *first, RzListIter *second, RzListComparator cmp, void *user) {
 	RzListIter *next = NULL, *result = NULL, *head = NULL;
 	while (first || second) {
 		if (!second) {
 			next = first;
-			first = first->n;
+			first = first->next;
 		} else if (!first) {
 			next = second;
-			second = second->n;
-		} else if (cmp(first->data, second->data) <= 0) {
+			second = second->next;
+		} else if (cmp(first->elem, second->elem, user) <= 0) {
 			next = first;
-			first = first->n;
+			first = first->next;
 		} else {
 			next = second;
-			second = second->n;
+			second = second->next;
 		}
 		if (!head) {
 			result = next;
 			head = result;
-			head->p = NULL;
+			head->prev = NULL;
 		} else {
-			result->n = next;
-			next->p = result;
-			result = result->n;
+			result->next = next;
+			next->prev = result;
+			result = result->next;
 		}
 	}
-	head->p = NULL;
-	next->n = NULL;
+	head->prev = NULL;
+	next->next = NULL;
 	return head;
 }
 
@@ -658,45 +694,45 @@ static RzListIter *_r_list_half_split(RzListIter *head) {
 	RzListIter *tmp;
 	RzListIter *fast;
 	RzListIter *slow;
-	if (!head || !head->n) {
+	if (!head || !head->next) {
 		return head;
 	}
 	slow = head;
 	fast = head;
-	while (fast && fast->n && fast->n->n) {
-		fast = fast->n->n;
-		slow = slow->n;
+	while (fast && fast->next && fast->next->next) {
+		fast = fast->next->next;
+		slow = slow->next;
 	}
-	tmp = slow->n;
-	slow->n = NULL;
+	tmp = slow->next;
+	slow->next = NULL;
 	return tmp;
 }
 
-static RzListIter *_merge_sort(RzListIter *head, RzListComparator cmp) {
+static RzListIter *_merge_sort(RzListIter *head, RzListComparator cmp, void *user) {
 	RzListIter *second;
-	if (!head || !head->n) {
+	if (!head || !head->next) {
 		return head;
 	}
 	second = _r_list_half_split(head);
-	head = _merge_sort(head, cmp);
-	second = _merge_sort(second, cmp);
-	return _merge(head, second, cmp);
+	head = _merge_sort(head, cmp, user);
+	second = _merge_sort(second, cmp, user);
+	return _merge(head, second, cmp, user);
 }
 
 /**
  * \brief Merge sorts the list via the RzListComparator
  *
  **/
-RZ_API void rz_list_merge_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp) {
-	rz_return_if_fail(list);
+RZ_API void rz_list_merge_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp, void *user) {
+	rz_return_if_fail(list && cmp);
 
 	if (!list->sorted && list->head && cmp) {
 		RzListIter *iter;
-		list->head = _merge_sort(list->head, cmp);
+		list->head = _merge_sort(list->head, cmp, user);
 		// update tail reference
 		iter = list->head;
-		while (iter && iter->n) {
-			iter = iter->n;
+		while (iter && iter->next) {
+			iter = iter->next;
 		}
 		list->tail = iter;
 	}
@@ -707,37 +743,36 @@ RZ_API void rz_list_merge_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListCompara
  * \brief Insertion sorts the list via the RzListComparator
  *
  **/
-RZ_API void rz_list_insertion_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp) {
-	rz_return_if_fail(list);
+RZ_API void rz_list_insertion_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp, void *user) {
+	rz_return_if_fail(list && cmp);
 
-	if (!list->sorted) {
-		RzListIter *it;
-		RzListIter *it2;
-		if (cmp) {
-			for (it = list->head; it && it->data; it = it->n) {
-				for (it2 = it->n; it2 && it2->data; it2 = it2->n) {
-					if (cmp(it->data, it2->data) > 0) {
-						void *t = it->data;
-						it->data = it2->data;
-						it2->data = t;
-					}
-				}
+	if (list->sorted) {
+		return;
+	}
+	RzListIter *it;
+	RzListIter *it2;
+	for (it = list->head; it && it->elem; it = it->next) {
+		for (it2 = it->next; it2 && it2->elem; it2 = it2->next) {
+			if (cmp(it->elem, it2->elem, user) > 0) {
+				void *t = it->elem;
+				it->elem = it2->elem;
+				it2->elem = t;
 			}
 		}
-		list->sorted = true;
 	}
+	list->sorted = true;
 }
 
 /**
  * \brief Sorts via merge sort or via insertion sort a list
  *
  **/
-RZ_API void rz_list_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp) {
-	rz_return_if_fail(list);
+RZ_API void rz_list_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp, void *user) {
+	rz_return_if_fail(list && cmp);
 	if (list->length > 43) {
-		rz_list_merge_sort(list, cmp);
+		rz_list_merge_sort(list, cmp, user);
 	} else {
-		rz_list_insertion_sort(list, cmp);
+		rz_list_insertion_sort(list, cmp, user);
 	}
 }
 
@@ -745,12 +780,11 @@ RZ_API void rz_list_sort(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cm
  * \brief Returns a new RzList which contains only unique values
  *
  **/
-RZ_API RZ_OWN RzList *rz_list_uniq(RZ_NONNULL const RzList *list, RZ_NONNULL RzListComparator cmp) {
-	RzListIter *iter, *iter2;
-	void *item, *item2;
-
+RZ_API RZ_OWN RzList *rz_list_uniq(RZ_NONNULL const RzList *list, RZ_NONNULL RzListComparator cmp, void *user) {
 	rz_return_val_if_fail(list && cmp, NULL);
 
+	RzListIter *iter, *iter2;
+	void *item, *item2;
 	RzList *nl = rz_list_newf(NULL);
 	if (!nl) {
 		return NULL;
@@ -758,7 +792,7 @@ RZ_API RZ_OWN RzList *rz_list_uniq(RZ_NONNULL const RzList *list, RZ_NONNULL RzL
 	rz_list_foreach (list, iter, item) {
 		bool found = false;
 		rz_list_foreach (nl, iter2, item2) {
-			if (cmp(item, item2) == 0) {
+			if (cmp(item, item2, user) == 0) {
 				found = true;
 				break;
 			}
@@ -770,9 +804,32 @@ RZ_API RZ_OWN RzList *rz_list_uniq(RZ_NONNULL const RzList *list, RZ_NONNULL RzL
 	return nl;
 }
 
+/**
+ * \brief Removes duplicate values from a sorted list in-place.
+ *
+ * Use only on a list that is sorted with respect to the RzListComparator.
+ **/
+RZ_API void rz_list_sorted_uniq(RZ_NONNULL RzList *list, RZ_NONNULL RzListComparator cmp, void *user) {
+	rz_return_if_fail(list && cmp);
+
+	RzListIter *iter, *tmp_iter;
+	void *cur, *prev = NULL;
+	rz_list_foreach_safe (list, iter, tmp_iter, cur) {
+		if (prev && cmp(prev, cur, user) == 0) {
+			rz_list_delete(list, iter);
+			continue;
+		}
+		prev = cur;
+	}
+}
+
 /**
  * \brief Casts a RzList containg strings into a concatenated string
  *
+ * \param list The list of strings to concatenate.
+ * \param ch char to separate the match strings.
+ *
+ * \return The concatenated string.
  **/
 RZ_API RZ_OWN char *rz_list_to_str(RZ_NONNULL RzList *list, char ch) {
 	RzListIter *iter;
@@ -786,17 +843,3 @@ RZ_API RZ_OWN char *rz_list_to_str(RZ_NONNULL RzList *list, char ch) {
 	}
 	return rz_strbuf_drain(buf);
 }
-
-/**
- * \brief Converts a SdbList into a RzList
- *
- **/
-RZ_API RZ_OWN RzList *rz_list_of_sdblist(SdbList *sl) {
-	RzList *l = rz_list_newf(free);
-	SdbKv *kv;
-	SdbListIter *iter;
-	ls_foreach (sl, iter, kv) {
-		rz_list_append(l, strdup(sdbkv_key(kv)));
-	}
-	return l;
-}
