@@ -171,6 +171,7 @@ static void check_pop_return(MCInst *MI) {
 		cs_arm_op *op = &ARM_get_detail(MI)->operands[i];
 		if (op->type == ARM_OP_REG && op->reg == ARM_REG_PC) {
 			add_group(MI, ARM_GRP_RET);
+			return;
 		}
 	}
 }
@@ -225,7 +226,12 @@ static void add_alias_details(MCInst *MI) {
 		return;
 	case ARM_INS_ALIAS_POP:
 		// Doesn't get set because memop is not printed.
-		ARM_get_detail(MI)->post_index = true;
+		if (ARM_get_detail(MI)->op_count == 1) {
+			CS_ASSERT_RET(MI->flat_insn->usesAliasDetails && "Not valid assumption for non alias details.");
+			// Only single register pop is post-indexed
+			// Assumes only alias details are passed here.
+			ARM_get_detail(MI)->post_index = true;
+		}
 		// fallthrough
 	case ARM_INS_ALIAS_PUSH:
 	case ARM_INS_ALIAS_VPUSH:
@@ -246,6 +252,39 @@ static void add_alias_details(MCInst *MI) {
 		}
 		break;
 	}
+	case ARM_INS_ALIAS_ASR:
+	case ARM_INS_ALIAS_LSL:
+	case ARM_INS_ALIAS_LSR:
+	case ARM_INS_ALIAS_ROR: {
+		unsigned shift_value = 0;
+		arm_shifter shift_type = ARM_SFT_INVALID;
+		switch (MCInst_getOpcode(MI)) {
+		default:
+			CS_ASSERT_RET(0 && "ASR, LSL, LSR, ROR alias not handled");
+			return;
+		case ARM_MOVsi: {
+			MCOperand *MO2 = MCInst_getOperand(MI, 2);
+			shift_type = (arm_shifter) ARM_AM_getSORegShOp(MCOperand_getImm(MO2));
+
+			if (ARM_AM_getSORegShOp(MCOperand_getImm(MO2)) == ARM_AM_rrx) {
+				break;
+			}
+			shift_value = translateShiftImm(ARM_AM_getSORegOffset(
+							       MCOperand_getImm(MO2)));
+			ARM_insert_detail_op_imm_at(MI, -1, shift_value, CS_AC_READ);
+			break;
+		}
+		case ARM_MOVsr: {
+			MCOperand *MO3 = MCInst_getOperand(MI, (3));
+			shift_type = ARM_AM_getSORegShOp(MCOperand_getImm(MO3)) + ARM_SFT_REG;
+			shift_value = MCInst_getOpVal(MI, 2);
+			break;
+		}
+		}
+		ARM_get_detail_op(MI, -2)->shift.type = shift_type;
+		ARM_get_detail_op(MI, -2)->shift.value = shift_value;
+		break;
+	}
 	}
 }
 
@@ -266,8 +305,8 @@ static void ARM_add_not_defined_ops(MCInst *MI)
 	switch (Opcode) {
 	default:
 		return;
-	case ARM_t2MOVsra_flag:
-	case ARM_t2MOVsrl_flag:
+	case ARM_t2MOVsra_glue:
+	case ARM_t2MOVsrl_glue:
 		ARM_insert_detail_op_imm_at(MI, 2, 1, CS_AC_READ);
 		break;
 	case ARM_VCMPEZD:
@@ -277,7 +316,7 @@ static void ARM_add_not_defined_ops(MCInst *MI)
 	case ARM_VCMPEZS:
 	case ARM_VCMPZH:
 	case ARM_VCMPZS:
-		ARM_insert_detail_op_imm_at(MI, 1, 0, CS_AC_READ);
+		ARM_insert_detail_op_imm_at(MI, -1, 0, CS_AC_READ);
 		break;
 	case ARM_MVE_VSHLL_lws16bh:
 	case ARM_MVE_VSHLL_lws16th:
@@ -494,8 +533,7 @@ static void ARM_add_not_defined_ops(MCInst *MI)
 		// Add shift information
 		ARM_get_detail(MI)->operands[1].shift.type =
 			(arm_shifter)ARM_AM_getSORegShOp(
-				MCInst_getOpVal(MI, 3)) +
-			ARM_SFT_ASR_REG - 1;
+				MCInst_getOpVal(MI, 3)) + ARM_SFT_REG;
 		ARM_get_detail(MI)->operands[1].shift.value =
 			MCInst_getOpVal(MI, 2);
 		break;
@@ -527,11 +565,25 @@ static void ARM_add_not_defined_ops(MCInst *MI)
 		}
 		break;
 	}
+	case ARM_RFEDA_UPD:
+	case ARM_RFEDB_UPD:
+	case ARM_RFEIA_UPD:
+	case ARM_RFEIB_UPD:
+		get_detail(MI)->writeback = true;
+		// fallthrough
+	case ARM_RFEDA:
+	case ARM_RFEDB:
+	case ARM_RFEIA:
+	case ARM_RFEIB: {
+		arm_reg base_reg = ARM_get_detail_op(MI, -1)->reg;
+		ARM_get_detail_op(MI, -1)->type = ARM_OP_MEM;
+		ARM_get_detail_op(MI, -1)->mem.base = base_reg;
+	}
 	}
 }
 
 /// Unfortunately there is currently no way to easily extract
-/// informaion about the vector data usage (sign and width used).
+/// information about the vector data usage (sign and width used).
 /// See: https://github.com/capstone-engine/capstone/issues/2152
 void ARM_add_vector_data(MCInst *MI, arm_vectordata_type data_type)
 {
@@ -541,7 +593,7 @@ void ARM_add_vector_data(MCInst *MI, arm_vectordata_type data_type)
 }
 
 /// Unfortunately there is currently no way to easily extract
-/// informaion about the vector size.
+/// information about the vector size.
 /// See: https://github.com/capstone-engine/capstone/issues/2152
 void ARM_add_vector_size(MCInst *MI, unsigned size)
 {
@@ -589,6 +641,26 @@ static void ARM_post_index_detection(MCInst *MI)
 	ARM_dec_op_count(MI);
 }
 
+void ARM_check_mem_access_validity(MCInst *MI)
+{
+#ifndef CAPSTONE_DIET
+	if (!detail_is_set(MI))
+		return;
+	const arm_suppl_info *suppl = map_get_suppl_info(MI, arm_insns);
+	CS_ASSERT_RET(suppl);
+	if (suppl->mem_acc == CS_AC_INVALID) {
+		return;
+	}
+	cs_detail *detail = get_detail(MI);
+	for (int i = 0; i < detail->arm.op_count; ++i) {
+		if (detail->arm.operands[i].type == ARM_OP_MEM && detail->arm.operands[i].access != suppl->mem_acc) {
+			detail->arm.operands[i].access = suppl->mem_acc;
+			return;
+		}
+	}
+#endif // CAPSTONE_DIET
+}
+
 /// Decodes the asm string for a given instruction
 /// and fills the detail information about the instruction and its operands.
 void ARM_printer(MCInst *MI, SStream *O, void * /* MCRegisterInfo* */ info)
@@ -601,6 +673,7 @@ void ARM_printer(MCInst *MI, SStream *O, void * /* MCRegisterInfo* */ info)
 	map_set_alias_id(MI, O, insn_alias_mnem_map, ARR_SIZE(insn_alias_mnem_map) - 1);
 	ARM_add_not_defined_ops(MI);
 	ARM_post_index_detection(MI);
+	ARM_check_mem_access_validity(MI);
 	ARM_add_cs_groups(MI);
 	int syntax_opt = MI->csh->syntax;
 	if (syntax_opt & CS_OPT_SYNTAX_CS_REG_ALIAS)
@@ -618,7 +691,7 @@ static const char *const insn_name_maps[] = {
 #endif
 
 #ifndef CAPSTONE_DIET
-static arm_reg arm_flag_regs[] = {
+static const arm_reg arm_flag_regs[] = {
 	ARM_REG_APSR,	      ARM_REG_APSR_NZCV, ARM_REG_CPSR,
 	ARM_REG_FPCXTNS,      ARM_REG_FPCXTS,	 ARM_REG_FPEXC,
 	ARM_REG_FPINST,	      ARM_REG_FPSCR,	 ARM_REG_FPSCR_NZCV,
@@ -762,11 +835,11 @@ void ARM_init_mri(MCRegisterInfo *MRI)
 					  ARR_SIZE(ARMSubRegIdxLists), 0);
 }
 
+#ifndef CAPSTONE_DIET
 static const map_insn_ops insn_operands[] = {
 #include "ARMGenCSMappingInsnOp.inc"
 };
 
-#ifndef CAPSTONE_DIET
 void ARM_reg_access(const cs_insn *insn, cs_regs regs_read,
 		    uint8_t *regs_read_count, cs_regs regs_write,
 		    uint8_t *regs_write_count)
@@ -935,6 +1008,7 @@ static void ARM_set_mem_access(MCInst *MI, bool status)
 #endif
 	} else {
 		// done, select the next operand slot
+		ARM_check_safe_inc(MI);
 		ARM_inc_op_count(MI);
 	}
 }
@@ -978,7 +1052,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	switch (op_group) {
 	default:
 		printf("ERROR: Operand group %d not handled!\n", op_group);
-		assert(0);
+		CS_ASSERT_RET(0);
 	case ARM_OP_GROUP_PredicateOperand:
 	case ARM_OP_GROUP_MandatoryPredicateOperand:
 	case ARM_OP_GROUP_MandatoryInvertedPredicateOperand:
@@ -1004,7 +1078,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_VPTPredicateOperand: {
 		ARMVCC_VPTCodes VCC = (ARMVCC_VPTCodes)MCOperand_getImm(
 			MCInst_getOperand(MI, OpNum));
-		assert(VCC <= ARMVCC_Else);
+		CS_ASSERT_RET(VCC <= ARMVCC_Else);
 		if (VCC != ARMVCC_None)
 			ARM_get_detail(MI)->vcc = VCC;
 		break;
@@ -1012,7 +1086,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_Operand:
 		if (op_type == CS_OP_IMM) {
 			if (doing_mem(MI)) {
-				ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+				ARM_set_detail_op_mem(MI, OpNum, false, 0,
 						      MCInst_getOpVal(MI,
 								      OpNum));
 			} else {
@@ -1026,14 +1100,14 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 				bool is_index_reg = map_get_op_type(MI, OpNum) &
 						    CS_OP_MEM;
 				ARM_set_detail_op_mem(
-					MI, OpNum, is_index_reg, 0, 0,
+					MI, OpNum, is_index_reg, is_index_reg ? 1 : 0,
 					MCInst_getOpVal(MI, OpNum));
 			} else {
 				ARM_set_detail_op_reg(
 					MI, OpNum, MCInst_getOpVal(MI, OpNum));
 			}
 		else
-			assert(0 && "Op type not handled.");
+			CS_ASSERT_RET(0 && "Op type not handled.");
 		break;
 	case ARM_OP_GROUP_PImmediate:
 		ARM_set_detail_op_imm(MI, OpNum, ARM_OP_PIMM,
@@ -1046,7 +1120,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_AddrMode6Operand:
 		if (!doing_mem(MI))
 			ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 		ARM_get_detail_op(MI, 0)->mem.align =
 			MCInst_getOpVal(MI, OpNum + 1) << 3;
@@ -1062,7 +1136,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_AddrMode7Operand:
 		if (!doing_mem(MI))
 			ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 		ARM_set_mem_access(MI, false);
 		break;
@@ -1160,6 +1234,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 			unsigned Reg =
 				MCOperand_getReg(MCInst_getOperand(MI, i));
 
+			ARM_check_safe_inc(MI);
 			ARM_get_detail_op(MI, 0)->type = ARM_OP_REG;
 			ARM_get_detail_op(MI, 0)->reg = Reg;
 			ARM_get_detail_op(MI, 0)->access = access;
@@ -1173,7 +1248,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		unsigned CondBit0 = Firstcond & 1;
 		unsigned NumTZ = CountTrailingZeros_32(Mask);
 		unsigned Pos, e;
-		ARM_PredBlockMask PredMask = 0;
+		ARM_PredBlockMask PredMask = ARM_PredBlockMaskInvalid;
 
 		// Check the documentation of ARM_PredBlockMask how the bits are set.
 		for (Pos = 3, e = NumTZ; Pos > e; --Pos) {
@@ -1192,7 +1267,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_VPTMask: {
 		unsigned Mask = MCInst_getOpVal(MI, OpNum);
 		unsigned NumTZ = CountTrailingZeros_32(Mask);
-		ARM_PredBlockMask PredMask = 0;
+		ARM_PredBlockMask PredMask = ARM_PredBlockMaskInvalid;
 
 		// Check the documentation of ARM_PredBlockMask how the bits are set.
 		for (unsigned Pos = 3, e = NumTZ; Pos > e; --Pos) {
@@ -1271,7 +1346,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		if (!SpecRegRBit && (Mask == 8 || Mask == 4 || Mask == 12)) {
 			switch (Mask) {
 			default:
-				assert(0 && "Unexpected mask value!");
+				CS_ASSERT_RET(0 && "Unexpected mask value!");
 			case 4:
 				ARM_set_detail_op_sysop(MI,
 							ARM_MCLASSSYSREG_APSR_G,
@@ -1319,7 +1394,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		int64_t imm =
 			MCOperand_getImm(MCInst_getOperand(MI, OpNum + 2));
 		ARM_get_detail_op(MI, 0)->shift.type =
-			(imm & 7) + ARM_SFT_ASR_REG - 1;
+			ARM_AM_getSORegShOp(imm) + ARM_SFT_REG;
 		if (ARM_AM_getSORegShOp(imm) != ARM_AM_rrx)
 			ARM_get_detail_op(MI, 0)->shift.value =
 				MCInst_getOpVal(MI, OpNum + 1);
@@ -1354,11 +1429,11 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 				      MCInst_getOpVal(MI, OpNum) + 1);
 		break;
 	case ARM_OP_GROUP_RotImmOperand: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		if (Imm == 0)
+		unsigned RotImm = MCInst_getOpVal(MI, OpNum);
+		if (RotImm == 0)
 			return;
 		ARM_get_detail_op(MI, -1)->shift.type = ARM_SFT_ROR;
-		ARM_get_detail_op(MI, -1)->shift.value = Imm * 8;
+		ARM_get_detail_op(MI, -1)->shift.value = RotImm * 8;
 		break;
 	}
 	case ARM_OP_GROUP_FBits16:
@@ -1390,30 +1465,29 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		break;
 	}
 	case ARM_OP_GROUP_PostIdxImm8Operand: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		bool sub = !(Imm & 256);
-		ARM_set_detail_op_mem_offset(MI, OpNum, (Imm & 0xff), sub);
+		unsigned Imm8 = MCInst_getOpVal(MI, OpNum);
+		bool sub = !(Imm8 & 256);
+		ARM_set_detail_op_mem_offset(MI, OpNum, (Imm8 & 0xff), sub);
 		ARM_get_detail(MI)->post_index = true;
 		break;
 	}
 	case ARM_OP_GROUP_PostIdxImm8s4Operand: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		bool sub = !(Imm & 256);
-		ARM_set_detail_op_mem_offset(MI, OpNum, (Imm & 0xff) << 2, sub);
+		unsigned Imm8s = MCInst_getOpVal(MI, OpNum);
+		bool sub = !(Imm8s & 256);
+		ARM_set_detail_op_mem_offset(MI, OpNum, (Imm8s & 0xff) << 2, sub);
 		ARM_get_detail(MI)->post_index = true;
 		break;
 	}
 	case ARM_OP_GROUP_AddrModeTBB:
 	case ARM_OP_GROUP_AddrModeTBH:
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
-		ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum + 1, true, 1,
 				      MCInst_getOpVal(MI, OpNum + 1));
 		if (op_group == ARM_OP_GROUP_AddrModeTBH) {
 			ARM_get_detail_op(MI, 0)->shift.type = ARM_SFT_LSL;
 			ARM_get_detail_op(MI, 0)->shift.value = 1;
-			ARM_get_detail_op(MI, 0)->mem.lshift = 1;
 		}
 		ARM_set_mem_access(MI, false);
 		break;
@@ -1424,23 +1498,20 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 			break;
 
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 		unsigned int imm3 = MCInst_getOpVal(MI, OpNum + 2);
 		unsigned ShOff = ARM_AM_getAM2Offset(imm3);
 		ARM_AM_AddrOpc subtracted = ARM_AM_getAM2Op(imm3);
 		if (!MCOperand_getReg(MCInst_getOperand(MI, OpNum + 1)) &&
 		    ShOff) {
-			ARM_get_detail_op(MI, 0)->shift.type =
-				(arm_shifter)subtracted;
 			ARM_get_detail_op(MI, 0)->shift.value = ShOff;
 			ARM_get_detail_op(MI, 0)->subtracted = subtracted ==
 							       ARM_AM_sub;
 			ARM_set_mem_access(MI, false);
 			break;
 		}
-		ARM_get_detail_op(MI, 0)->shift.type = subtracted == ARM_AM_sub;
-		ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum + 1, true, subtracted == ARM_AM_sub ? -1 : 1,
 				      MCInst_getOpVal(MI, OpNum + 1));
 		add_cs_detail_RegImmShift(MI, ARM_AM_getAM2ShiftOpc(imm3),
 					  ARM_AM_getAM2Offset(imm3));
@@ -1494,14 +1565,14 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 			break;
 
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 		unsigned ImmOffs = MCInst_getOpVal(MI, OpNum + 1);
 		if (ImmOffs) {
 			unsigned Scale = 0;
 			switch (op_group) {
 			default:
-				assert(0 &&
+				CS_ASSERT_RET(0 &&
 				       "Cannot determine scale. Operand group not handled.");
 			case ARM_OP_GROUP_ThumbAddrModeImm5S1Operand:
 				Scale = 1;
@@ -1514,7 +1585,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 				Scale = 4;
 				break;
 			}
-			ARM_set_detail_op_mem(MI, OpNum + 1, false, 0, 0,
+			ARM_set_detail_op_mem(MI, OpNum + 1, false, 0,
 					      ImmOffs * Scale);
 		}
 		ARM_set_mem_access(MI, false);
@@ -1527,11 +1598,11 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 			break;
 
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 		arm_reg RegNum = MCInst_getOpVal(MI, OpNum + 1);
 		if (RegNum)
-			ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+			ARM_set_detail_op_mem(MI, OpNum + 1, true, 1,
 					      RegNum);
 		ARM_set_mem_access(MI, false);
 		break;
@@ -1553,9 +1624,9 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		if (!doing_mem(MI))
 			ARM_set_mem_access(MI, true);
 
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
-		ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum + 1, true, 1,
 				      MCInst_getOpVal(MI, OpNum + 1));
 		unsigned ShAmt = MCInst_getOpVal(MI, OpNum + 2);
 		if (ShAmt) {
@@ -1567,28 +1638,28 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	}
 	case ARM_OP_GROUP_T2AddrModeImm0_1020s4Operand:
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
-		int64_t Imm = MCInst_getOpVal(MI, OpNum + 1);
-		if (Imm)
-			ARM_set_detail_op_mem(MI, OpNum + 1, false, 0, 0,
-					      Imm * 4);
+		int64_t Imm0_1024s4 = MCInst_getOpVal(MI, OpNum + 1);
+		if (Imm0_1024s4)
+			ARM_set_detail_op_mem(MI, OpNum + 1, false, 0,
+					      Imm0_1024s4 * 4);
 		ARM_set_mem_access(MI, false);
 		break;
 	case ARM_OP_GROUP_PKHLSLShiftImm: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		if (Imm == 0)
+		unsigned ShiftImm = MCInst_getOpVal(MI, OpNum);
+		if (ShiftImm == 0)
 			return;
 		ARM_get_detail_op(MI, -1)->shift.type = ARM_SFT_LSL;
-		ARM_get_detail_op(MI, -1)->shift.value = Imm;
+		ARM_get_detail_op(MI, -1)->shift.value = ShiftImm;
 		break;
 	}
 	case ARM_OP_GROUP_PKHASRShiftImm: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		if (Imm == 0)
-			Imm = 32;
+		unsigned RShiftImm = MCInst_getOpVal(MI, OpNum);
+		if (RShiftImm == 0)
+			RShiftImm = 32;
 		ARM_get_detail_op(MI, -1)->shift.type = ARM_SFT_ASR;
-		ARM_get_detail_op(MI, -1)->shift.value = Imm;
+		ARM_get_detail_op(MI, -1)->shift.value = RShiftImm;
 		break;
 	}
 	case ARM_OP_GROUP_ThumbS4ImmOperand:
@@ -1596,9 +1667,9 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 				      MCInst_getOpVal(MI, OpNum) * 4);
 		break;
 	case ARM_OP_GROUP_ThumbSRImm: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
+		unsigned SRImm = MCInst_getOpVal(MI, OpNum);
 		ARM_set_detail_op_imm(MI, OpNum, ARM_OP_IMM,
-				      Imm == 0 ? 32 : Imm);
+				      SRImm == 0 ? 32 : SRImm);
 		break;
 	}
 	case ARM_OP_GROUP_BitfieldInvMaskImmOperand: {
@@ -1610,8 +1681,8 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		break;
 	}
 	case ARM_OP_GROUP_CPSIMod: {
-		unsigned Imm = MCInst_getOpVal(MI, OpNum);
-		ARM_get_detail(MI)->cps_mode = Imm;
+		unsigned Mode = MCInst_getOpVal(MI, OpNum);
+		ARM_get_detail(MI)->cps_mode = Mode;
 		break;
 	}
 	case ARM_OP_GROUP_CPSIFlag: {
@@ -1661,6 +1732,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 		int32_t OffImm = MCInst_getOpVal(MI, OpNum);
 		if (OffImm == INT32_MIN)
 			OffImm = 0;
+		ARM_check_safe_inc(MI);
 		ARM_get_detail_op(MI, 0)->type = ARM_OP_MEM;
 		ARM_get_detail_op(MI, 0)->mem.base = ARM_REG_PC;
 		ARM_get_detail_op(MI, 0)->mem.index = ARM_REG_INVALID;
@@ -1683,6 +1755,7 @@ static void add_cs_detail_general(MCInst *MI, arm_op_group op_group,
 	}
 	case ARM_OP_GROUP_SetendOperand: {
 		bool be = MCInst_getOpVal(MI, OpNum) != 0;
+		ARM_check_safe_inc(MI);
 		if (be) {
 			ARM_get_detail_op(MI, 0)->type = ARM_OP_SETEND;
 			ARM_get_detail_op(MI, 0)->setend = ARM_SETEND_BE;
@@ -1713,7 +1786,7 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 	switch (op_group) {
 	default:
 		printf("ERROR: Operand group %d not handled!\n", op_group);
-		assert(0);
+		CS_ASSERT_RET(0);
 	case ARM_OP_GROUP_AddrModeImm12Operand_0:
 	case ARM_OP_GROUP_AddrModeImm12Operand_1:
 	case ARM_OP_GROUP_T2AddrModeImm8s4Operand_0:
@@ -1728,12 +1801,12 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_T2AddrModeImm8Operand_1: {
 		bool AlwaysPrintImm0 = temp_arg_0;
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
-		int32_t Imm = MCInst_getOpVal(MI, OpNum + 1);
-		if (Imm == INT32_MIN)
-			Imm = 0;
-		ARM_set_detail_op_mem(MI, OpNum + 1, false, 0, 0, Imm);
+		int32_t Imm8 = MCInst_getOpVal(MI, OpNum + 1);
+		if (Imm8 == INT32_MIN)
+			Imm8 = 0;
+		ARM_set_detail_op_mem(MI, OpNum + 1, false, 0, Imm8);
 		if (AlwaysPrintImm0)
 			map_add_implicit_write(MI, MCInst_getOpVal(MI, OpNum));
 
@@ -1758,7 +1831,7 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 			break;
 
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
 
 		MCOperand *MO2 = MCInst_getOperand(MI, OpNum + 1);
@@ -1766,7 +1839,7 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 			ARM_AM_getAM3Op(MCInst_getOpVal(MI, OpNum + 2));
 
 		if (MCOperand_getReg(MO2)) {
-			ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+			ARM_set_detail_op_mem(MI, OpNum + 1, true, Sign == ARM_AM_sub ? -1 : 1,
 					      MCInst_getOpVal(MI, OpNum + 1));
 			ARM_get_detail_op(MI, 0)->subtracted = Sign ==
 							       ARM_AM_sub;
@@ -1777,7 +1850,7 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 			ARM_AM_getAM3Offset(MCInst_getOpVal(MI, OpNum + 2));
 
 		if (AlwaysPrintImm0 || ImmOffs || Sign == ARM_AM_sub) {
-			ARM_set_detail_op_mem(MI, OpNum + 2, false, 0, 0,
+			ARM_set_detail_op_mem(MI, OpNum + 2, false, 0,
 					      ImmOffs);
 			ARM_get_detail_op(MI, 0)->subtracted = Sign ==
 							       ARM_AM_sub;
@@ -1790,9 +1863,12 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_AddrMode5FP16Operand_0: {
 		bool AlwaysPrintImm0 = temp_arg_0;
 
-		if (AlwaysPrintImm0)
+		if (AlwaysPrintImm0) {
+			get_detail(MI)->writeback = true;
 			map_add_implicit_write(MI, MCInst_getOpVal(MI, OpNum));
+		}
 
+		ARM_check_safe_inc(MI);
 		cs_arm_op *Op = ARM_get_detail_op(MI, 0);
 		Op->type = ARM_OP_MEM;
 		Op->mem.base = MCInst_getOpVal(MI, OpNum);
@@ -1823,9 +1899,9 @@ static void add_cs_detail_template_1(MCInst *MI, arm_op_group op_group,
 	case ARM_OP_GROUP_MveAddrModeRQOperand_3: {
 		unsigned Shift = temp_arg_0;
 		ARM_set_mem_access(MI, true);
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum, false, 0,
 				      MCInst_getOpVal(MI, OpNum));
-		ARM_set_detail_op_mem(MI, OpNum + 1, true, 0, 0,
+		ARM_set_detail_op_mem(MI, OpNum + 1, true, 1,
 				      MCInst_getOpVal(MI, OpNum + 1));
 		if (Shift > 0) {
 			add_cs_detail_RegImmShift(MI, ARM_AM_uxtw, Shift);
@@ -1859,13 +1935,13 @@ static void add_cs_detail_template_2(MCInst *MI, arm_op_group op_group,
 	switch (op_group) {
 	default:
 		printf("ERROR: Operand group %d not handled!\n", op_group);
-		assert(0);
+		CS_ASSERT_RET(0);
 	case ARM_OP_GROUP_ComplexRotationOp_90_0:
 	case ARM_OP_GROUP_ComplexRotationOp_180_90: {
 		unsigned Angle = temp_arg_0;
 		unsigned Remainder = temp_arg_1;
-		unsigned Imm = (MCInst_getOpVal(MI, OpNum) * Angle) + Remainder;
-		ARM_set_detail_op_imm(MI, OpNum, ARM_OP_IMM, Imm);
+		unsigned Rotation = (MCInst_getOpVal(MI, OpNum) * Angle) + Remainder;
+		ARM_set_detail_op_imm(MI, OpNum, ARM_OP_IMM, Rotation);
 		break;
 	}
 	}
@@ -1924,25 +2000,20 @@ void ARM_add_cs_detail(MCInst *MI, int /* arm_op_group */ op_group,
 	add_cs_detail_general(MI, op_group, op_num);
 }
 
-/// Inserts a register to the detail operands at @index.
-/// Already present operands are moved.
-void ARM_insert_detail_op_reg_at(MCInst *MI, unsigned index, arm_reg Reg,
-				 cs_ac_type access)
+static void insert_op(MCInst *MI, unsigned index, cs_arm_op op)
 {
-	if (!detail_is_set(MI))
+	if (!detail_is_set(MI)) {
 		return;
-
-	assert(ARM_get_detail(MI)->op_count < MAX_ARM_OPS);
-
-	cs_arm_op op;
-	ARM_setup_op(&op);
-	op.type = ARM_OP_REG;
-	op.reg = Reg;
-	op.access = access;
+	}
+	ARM_check_safe_inc(MI);
 
 	cs_arm_op *ops = ARM_get_detail(MI)->operands;
 	int i = ARM_get_detail(MI)->op_count;
-	assert(i < MAX_ARM_OPS);
+	if (index == -1) {
+		ops[i] = op;
+		ARM_inc_op_count(MI);
+		return;
+	}
 	for (; i > 0 && i > index; --i) {
 		ops[i] = ops[i - 1];
 	}
@@ -1950,15 +2021,32 @@ void ARM_insert_detail_op_reg_at(MCInst *MI, unsigned index, arm_reg Reg,
 	ARM_inc_op_count(MI);
 }
 
-/// Inserts a immediate to the detail operands at @index.
+/// Inserts a register to the detail operands at @index.
 /// Already present operands are moved.
-void ARM_insert_detail_op_imm_at(MCInst *MI, unsigned index, int64_t Val,
+/// If @index is -1 the operand is appended.
+void ARM_insert_detail_op_reg_at(MCInst *MI, unsigned index, arm_reg Reg,
 				 cs_ac_type access)
 {
 	if (!detail_is_set(MI))
 		return;
 
-	assert(ARM_get_detail(MI)->op_count < MAX_ARM_OPS);
+	cs_arm_op op;
+	ARM_setup_op(&op);
+	op.type = ARM_OP_REG;
+	op.reg = Reg;
+	op.access = access;
+	insert_op(MI, index, op);
+}
+
+/// Inserts a immediate to the detail operands at @index.
+/// Already present operands are moved.
+/// If @index is -1 the operand is appended.
+void ARM_insert_detail_op_imm_at(MCInst *MI, unsigned index, int64_t Val,
+				 cs_ac_type access)
+{
+	if (!detail_is_set(MI))
+		return;
+	ARM_check_safe_inc(MI);
 
 	cs_arm_op op;
 	ARM_setup_op(&op);
@@ -1966,14 +2054,7 @@ void ARM_insert_detail_op_imm_at(MCInst *MI, unsigned index, int64_t Val,
 	op.imm = Val;
 	op.access = access;
 
-	cs_arm_op *ops = ARM_get_detail(MI)->operands;
-	int i = ARM_get_detail(MI)->op_count;
-	assert(i < MAX_ARM_OPS);
-	for (; i > 0 && i > index; --i) {
-		ops[i] = ops[i - 1];
-	}
-	ops[index] = op;
-	ARM_inc_op_count(MI);
+	insert_op(MI, index, op);
 }
 
 /// Adds a register ARM operand at position OpNum and increases the op_count by
@@ -1982,8 +2063,9 @@ void ARM_set_detail_op_reg(MCInst *MI, unsigned OpNum, arm_reg Reg)
 {
 	if (!detail_is_set(MI))
 		return;
-	assert(!(map_get_op_type(MI, OpNum) & CS_OP_MEM));
-	assert(map_get_op_type(MI, OpNum) == CS_OP_REG);
+	ARM_check_safe_inc(MI);
+	CS_ASSERT_RET(!(map_get_op_type(MI, OpNum) & CS_OP_MEM));
+	CS_ASSERT_RET(map_get_op_type(MI, OpNum) == CS_OP_REG);
 
 	ARM_get_detail_op(MI, 0)->type = ARM_OP_REG;
 	ARM_get_detail_op(MI, 0)->reg = Reg;
@@ -1998,9 +2080,10 @@ void ARM_set_detail_op_imm(MCInst *MI, unsigned OpNum, arm_op_type ImmType,
 {
 	if (!detail_is_set(MI))
 		return;
-	assert(!(map_get_op_type(MI, OpNum) & CS_OP_MEM));
-	assert(map_get_op_type(MI, OpNum) == CS_OP_IMM);
-	assert(ImmType == ARM_OP_IMM || ImmType == ARM_OP_PIMM ||
+	ARM_check_safe_inc(MI);
+	CS_ASSERT_RET(!(map_get_op_type(MI, OpNum) & CS_OP_MEM));
+	CS_ASSERT_RET(map_get_op_type(MI, OpNum) == CS_OP_IMM);
+	CS_ASSERT_RET(ImmType == ARM_OP_IMM || ImmType == ARM_OP_PIMM ||
 	       ImmType == ARM_OP_CIMM);
 
 	ARM_get_detail_op(MI, 0)->type = ImmType;
@@ -2013,20 +2096,20 @@ void ARM_set_detail_op_imm(MCInst *MI, unsigned OpNum, arm_op_type ImmType,
 void ARM_set_detail_op_mem_offset(MCInst *MI, unsigned OpNum, uint64_t Val,
 				  bool subtracted)
 {
-	assert(map_get_op_type(MI, OpNum) & CS_OP_MEM);
+	CS_ASSERT_RET(map_get_op_type(MI, OpNum) & CS_OP_MEM);
 
 	if (!doing_mem(MI)) {
-		assert((ARM_get_detail_op(MI, -1) != NULL) &&
+		CS_ASSERT_RET((ARM_get_detail_op(MI, -1) != NULL) &&
 		       (ARM_get_detail_op(MI, -1)->type == ARM_OP_MEM));
 		ARM_dec_op_count(MI);
 	}
 
 	if ((map_get_op_type(MI, OpNum) & ~CS_OP_MEM) == CS_OP_IMM)
-		ARM_set_detail_op_mem(MI, OpNum, false, 0, 0, Val);
+		ARM_set_detail_op_mem(MI, OpNum, false, 0, Val);
 	else if ((map_get_op_type(MI, OpNum) & ~CS_OP_MEM) == CS_OP_REG)
-		ARM_set_detail_op_mem(MI, OpNum, true, 0, 0, Val);
+		ARM_set_detail_op_mem(MI, OpNum, true, subtracted ? -1 : 1, Val);
 	else
-		assert(0 && "Memory type incorrect.");
+		CS_ASSERT_RET(0 && "Memory type incorrect.");
 	ARM_get_detail_op(MI, 0)->subtracted = subtracted;
 
 	if (!doing_mem(MI))
@@ -2036,17 +2119,17 @@ void ARM_set_detail_op_mem_offset(MCInst *MI, unsigned OpNum, uint64_t Val,
 /// Adds a memory ARM operand at position OpNum. op_count is *not* increased by
 /// one. This is done by ARM_set_mem_access().
 void ARM_set_detail_op_mem(MCInst *MI, unsigned OpNum, bool is_index_reg,
-			   int scale, int lshift, uint64_t Val)
+			   int scale, uint64_t Val)
 {
 	if (!detail_is_set(MI))
 		return;
-	assert(map_get_op_type(MI, OpNum) & CS_OP_MEM);
+	CS_ASSERT_RET(map_get_op_type(MI, OpNum) & CS_OP_MEM);
 	cs_op_type secondary_type = map_get_op_type(MI, OpNum) & ~CS_OP_MEM;
 	switch (secondary_type) {
 	default:
-		assert(0 && "Secondary type not supported yet.");
+		CS_ASSERT_RET(0 && "Secondary type not supported yet.");
 	case CS_OP_REG: {
-		assert(secondary_type == CS_OP_REG);
+		CS_ASSERT_RET(secondary_type == CS_OP_REG);
 		if (!is_index_reg) {
 			ARM_get_detail_op(MI, 0)->mem.base = Val;
 			if (MCInst_opIsTying(MI, OpNum) || MCInst_opIsTied(MI, OpNum)) {
@@ -2069,12 +2152,11 @@ void ARM_set_detail_op_mem(MCInst *MI, unsigned OpNum, bool is_index_reg,
 			ARM_get_detail_op(MI, 0)->mem.index = Val;
 		}
 		ARM_get_detail_op(MI, 0)->mem.scale = scale;
-		ARM_get_detail_op(MI, 0)->mem.lshift = lshift;
 
 		break;
 	}
 	case CS_OP_IMM: {
-		assert(secondary_type == CS_OP_IMM);
+		CS_ASSERT_RET(secondary_type == CS_OP_IMM);
 		if (((int32_t)Val) < 0)
 			ARM_get_detail_op(MI, 0)->subtracted = true;
 		ARM_get_detail_op(MI, 0)->mem.disp = ((int64_t)Val < 0) ? -Val :
@@ -2093,7 +2175,7 @@ void ARM_set_detail_op_neon_lane(MCInst *MI, unsigned OpNum)
 {
 	if (!detail_is_set(MI))
 		return;
-	assert(map_get_op_type(MI, OpNum) == CS_OP_IMM);
+	CS_ASSERT_RET(map_get_op_type(MI, OpNum) == CS_OP_IMM);
 	unsigned Val = MCOperand_getImm(MCInst_getOperand(MI, OpNum));
 
 	ARM_get_detail_op(MI, -1)->neon_lane = Val;
@@ -2107,12 +2189,14 @@ void ARM_set_detail_op_sysop(MCInst *MI, int Val, arm_op_type type,
 {
 	if (!detail_is_set(MI))
 		return;
+	ARM_check_safe_inc(MI);
+
 	ARM_get_detail_op(MI, 0)->type = type;
 	switch (type) {
 	default:
-		assert(0 && "Unkown system operand type.");
+		CS_ASSERT_RET(0 && "Unknown system operand type.");
 	case ARM_OP_SYSREG:
-		ARM_get_detail_op(MI, 0)->sysop.reg.mclasssysreg = Val;
+		ARM_get_detail_op(MI, 0)->sysop.reg.mclasssysreg = Val; // NOLINT(clang-analyzer-optin.core.EnumCastOutOfRange)
 		break;
 	case ARM_OP_BANKEDREG:
 		ARM_get_detail_op(MI, 0)->sysop.reg.bankedreg = Val;
@@ -2121,7 +2205,7 @@ void ARM_set_detail_op_sysop(MCInst *MI, int Val, arm_op_type type,
 	case ARM_OP_CPSR:
 		ARM_get_detail_op(MI, 0)->reg =
 			type == ARM_OP_SPSR ? ARM_REG_SPSR : ARM_REG_CPSR;
-		ARM_get_detail_op(MI, 0)->sysop.psr_bits = Val;
+		ARM_get_detail_op(MI, 0)->sysop.psr_bits = Val; // NOLINT(clang-analyzer-optin.core.EnumCastOutOfRange)
 		break;
 	}
 	ARM_get_detail_op(MI, 0)->sysop.sysm = Sysm;
@@ -2136,6 +2220,8 @@ void ARM_set_detail_op_float(MCInst *MI, unsigned OpNum, uint64_t Imm)
 {
 	if (!detail_is_set(MI))
 		return;
+	ARM_check_safe_inc(MI);
+
 	ARM_get_detail_op(MI, 0)->type = ARM_OP_FP;
 	ARM_get_detail_op(MI, 0)->fp = ARM_AM_getFPImmFloat(Imm);
 	ARM_inc_op_count(MI);
