@@ -5,7 +5,6 @@
 
 #include "../../utils.h"
 #include "../../MCRegisterInfo.h"
-#include "PPCInstPrinter.h"
 #include "PPCMapping.h"
 #include "PPCModule.h"
 
@@ -35,7 +34,14 @@ cs_err PPC_option(cs_struct *handle, cs_opt_type type, size_t value)
 		handle->syntax = (int)value;
 
 	if (type == CS_OPT_MODE) {
+		if (value == CS_MODE_LITTLE_ENDIAN) {
+			handle->mode = handle->mode & ~CS_MODE_BIG_ENDIAN;
+			return CS_ERR_OK;
+		}
 		handle->mode |= (cs_mode)value;
+		if (value & CS_MODE_MSYNC) {
+			handle->mode |= (cs_mode)CS_MODE_BOOKE;
+		}
 	}
 
 	return CS_ERR_OK;
