@@ -31,7 +31,7 @@ case $1 in
     ARCH=SPARC
     ;;
   SystemZ)
-    ARCH=SYSZ
+    ARCH=SYSTEMZ
     ;;
   XCore)
     ARCH=XCORE
@@ -60,6 +60,15 @@ case $1 in
   RISCV)
     ARCH=RISCV
     ;;
+  HPPA)
+    ARCH=HPPA
+    ;;
+  LOONGARCH)
+    ARCH=LOONGARCH
+    ;;
+  ARC)
+    ARCH=ARC
+    ;;
   *)
     ;;
 esac
