@@ -117,7 +117,7 @@ static RzAnnotatedCode *get_hello_world(void) {
 	RzVector /*<RzCodeAnnotation>*/ *test_annotations;
 	test_annotations = get_annotations_for_hello_world();
 	RzCodeAnnotation *annotation;
-	rz_vector_foreach(test_annotations, annotation) {
+	rz_vector_foreach (test_annotations, annotation) {
 		rz_annotated_code_add_annotation(code, annotation);
 	}
 
@@ -191,7 +191,7 @@ static bool test_rz_annotated_code_add_annotation(void) {
 	RzVector /*<RzCodeAnnotation>*/ *test_annotations;
 	test_annotations = get_some_code_annotation_for_add();
 	RzCodeAnnotation *annotation;
-	rz_vector_foreach(test_annotations, annotation) {
+	rz_vector_foreach (test_annotations, annotation) {
 		rz_annotated_code_add_annotation(code, annotation);
 	}
 
@@ -215,20 +215,20 @@ static bool test_rz_annotated_code_annotations_in(void) {
 	test_annotations = get_some_annotations_for_in();
 
 	RzCodeAnnotation *annotation;
-	rz_vector_foreach(test_annotations, annotation) {
+	rz_vector_foreach (test_annotations, annotation) {
 		rz_annotated_code_add_annotation(code, annotation);
 	}
 
 	RzPVector *out = rz_annotated_code_annotations_in(code, 11);
 	// Expecting indices = 3, 4, 5
 	mu_assert_eq(out->v.len, 3, "Additional annotations found. Bad output.");
-	if (!test_equal(*rz_pvector_index_ptr(out, 0), rz_vector_index_ptr(test_annotations, 3))) {
+	if (!test_equal(rz_pvector_at(out, 0), rz_vector_index_ptr(test_annotations, 3))) {
 		return false;
 	}
-	if (!test_equal(*rz_pvector_index_ptr(out, 1), rz_vector_index_ptr(test_annotations, 4))) {
+	if (!test_equal(rz_pvector_at(out, 1), rz_vector_index_ptr(test_annotations, 4))) {
 		return false;
 	}
-	if (!test_equal(*rz_pvector_index_ptr(out, 2), rz_vector_index_ptr(test_annotations, 5))) {
+	if (!test_equal(rz_pvector_at(out, 2), rz_vector_index_ptr(test_annotations, 5))) {
 		return false;
 	}
 
@@ -244,23 +244,23 @@ static bool test_rz_annotated_code_annotations_range(void) {
 	RzVector /*<RzCodeAnnotation>*/ *test_annotations;
 	test_annotations = get_some_annotations_for_in();
 	RzCodeAnnotation *annotation;
-	rz_vector_foreach(test_annotations, annotation) {
+	rz_vector_foreach (test_annotations, annotation) {
 		rz_annotated_code_add_annotation(code, annotation);
 	}
 
 	RzPVector *out = rz_annotated_code_annotations_range(code, 7, 16);
 	// Expecting indices = 2, 3, 4, 5
 	mu_assert_eq(out->v.len, 4, "Additional annotations found. Bad output.");
-	if (!test_equal(*rz_pvector_index_ptr(out, 0), rz_vector_index_ptr(test_annotations, 2))) {
+	if (!test_equal(rz_pvector_at(out, 0), rz_vector_index_ptr(test_annotations, 2))) {
 		return false;
 	}
-	if (!test_equal(*rz_pvector_index_ptr(out, 1), rz_vector_index_ptr(test_annotations, 3))) {
+	if (!test_equal(rz_pvector_at(out, 1), rz_vector_index_ptr(test_annotations, 3))) {
 		return false;
 	}
-	if (!test_equal(*rz_pvector_index_ptr(out, 2), rz_vector_index_ptr(test_annotations, 4))) {
+	if (!test_equal(rz_pvector_at(out, 2), rz_vector_index_ptr(test_annotations, 4))) {
 		return false;
 	}
-	if (!test_equal(*rz_pvector_index_ptr(out, 3), rz_vector_index_ptr(test_annotations, 5))) {
+	if (!test_equal(rz_pvector_at(out, 3), rz_vector_index_ptr(test_annotations, 5))) {
 		return false;
 	}
 
