@@ -16,6 +16,16 @@ static RzCmdDescHelp fake_help = {
 	.args = fake_args,
 };
 
+static RzCmdDescArg fake_args_1[] = {
+	{ .name = "fake_arg", .type = RZ_CMD_ARG_TYPE_RAW },
+	{ 0 },
+};
+
+static RzCmdDescHelp fake_help_1 = {
+	.summary = "fake help",
+	.args = fake_args_1,
+};
+
 bool test_parsed_args_noargs(void) {
 	RzCmdParsedArgs *a = rz_cmd_parsed_args_new("pd", 0, NULL);
 	mu_assert_streq(a->argv[0], "pd", "pd is the command");
@@ -121,23 +131,6 @@ bool test_cmd_descriptor_argv_nested(void) {
 	mu_end;
 }
 
-static int a_oldinput_cb(void *user, const char *input) {
-	return 0;
-}
-
-bool test_cmd_descriptor_oldinput(void) {
-	RzCmd *cmd = rz_cmd_new(NULL, false);
-	RzCmdDesc *root = rz_cmd_get_root(cmd);
-	RzCmdDesc *cd = rz_cmd_desc_oldinput_new(cmd, root, "a", a_oldinput_cb, NULL);
-	mu_assert_notnull(cd, "cmddesc created");
-	mu_assert_streq(cd->name, "a", "command descriptor name is a");
-	mu_assert_eq(cd->type, RZ_CMD_DESC_TYPE_OLDINPUT, "type of command descriptor is oldinput");
-	mu_assert_ptreq(rz_cmd_desc_parent(cd), root, "root parent descriptor");
-	mu_assert_eq(cd->n_children, 0, "no children");
-	rz_cmd_free(cmd);
-	mu_end;
-}
-
 static RzCmdStatus a_exec_cb(RzCore *core, int argc, const char **argv) {
 	return RZ_CMD_STATUS_OK;
 }
@@ -192,12 +185,12 @@ static RzCmdStatus aeir_handler(RzCore *core, int argc, const char **argv) {
 	return RZ_CMD_STATUS_OK;
 }
 
-static int ae_handler(void *user, const char *input) {
-	return 0;
+static RzCmdStatus ae_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_OK;
 }
 
-static int w_handler(void *user, const char *input) {
-	return 0;
+static RzCmdStatus w_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_OK;
 }
 
 bool test_cmd_descriptor_tree(void) {
@@ -205,7 +198,7 @@ bool test_cmd_descriptor_tree(void) {
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, a_cd, "ap", ap_handler, &fake_help);
-	rz_cmd_desc_oldinput_new(cmd, root, "w", w_handler, NULL);
+	rz_cmd_desc_group_new(cmd, root, "w", w_handler, NULL, NULL);
 
 	void **it_cd;
 	rz_cmd_desc_children_foreach(root, it_cd) {
@@ -223,9 +216,9 @@ bool test_cmd_get_desc(void) {
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	RzCmdDesc *ap_cd = rz_cmd_desc_group_new(cmd, a_cd, "ap", ap_handler, NULL, &fake_help);
 	RzCmdDesc *apd_cd = rz_cmd_desc_argv_new(cmd, ap_cd, "apd", ap_handler, &fake_help);
-	RzCmdDesc *ae_cd = rz_cmd_desc_oldinput_new(cmd, a_cd, "ae", ae_handler, NULL);
+	RzCmdDesc *ae_cd = rz_cmd_desc_argv_new(cmd, a_cd, "ae", ae_handler, NULL);
 	RzCmdDesc *aeir_cd = rz_cmd_desc_argv_new(cmd, ae_cd, "aeir", aeir_handler, &fake_help);
-	RzCmdDesc *w_cd = rz_cmd_desc_oldinput_new(cmd, root, "w", w_handler, NULL);
+	RzCmdDesc *w_cd = rz_cmd_desc_argv_new(cmd, root, "w", w_handler, NULL);
 
 	mu_assert_null(rz_cmd_get_desc(cmd, "afl"), "afl does not have any handler");
 	mu_assert_ptreq(rz_cmd_get_desc(cmd, "ap"), ap_cd, "ap will be handled by ap");
@@ -253,25 +246,20 @@ static RzCmdStatus p_handler_argv(RzCore *core, int argc, const char **argv) {
 	return RZ_CMD_STATUS_OK;
 }
 
-static int p_handler(void *user, const char *input) {
-	mu_assert_streq(input, "x 10", "input is +1");
-	return -1;
+static RzCmdStatus p_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_ERROR;
 }
 
-static int px_handler(void *user, const char *input) {
-	if (*input == '?') {
-		rz_cons_printf("Free format px help\n");
-	}
-	return 0;
+static RzCmdStatus px_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_OK;
 }
 
-static int wv_handler(void *user, const char *input) {
-	mu_assert_streq(input, "8 0xdeadbeef", "input is +2");
-	return 1;
+static RzCmdStatus wv_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_OK;
 }
 
-static int q_handler(void *user, const char *input) {
-	return -2;
+static RzCmdStatus q_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_EXIT;
 }
 
 bool test_cmd_call_desc(void) {
@@ -285,15 +273,15 @@ bool test_cmd_call_desc(void) {
 
 	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
-	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
+	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", p_handler, NULL, &fake_help);
+	RzCmdDesc *w_cd = rz_cmd_desc_group_new(cmd, root, "w", w_handler, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
-	rz_cmd_desc_oldinput_new(cmd, p_cd, "p", p_handler, NULL);
-	rz_cmd_desc_oldinput_new(cmd, root, "wv", wv_handler, NULL);
-	rz_cmd_desc_oldinput_new(cmd, root, "q", q_handler, NULL);
+	mu_assert_notnull(rz_cmd_desc_argv_new(cmd, w_cd, "wv", wv_handler, &fake_help_1), "arg was not added.");
+	mu_assert_notnull(rz_cmd_desc_argv_new(cmd, root, "q", q_handler, &fake_help), "arg was not added.");
 
 	char *pd_args[] = { "10" };
 	char *px_args[] = { "10" };
-	char *wv8_args[] = { "0xdeadbeef" };
+	char *wv_args[] = { "0xdeadbeef" };
 
 	RzCmdParsedArgs *a = rz_cmd_parsed_args_new("pd", 1, pd_args);
 	mu_assert_eq(rz_cmd_call_parsed_args(cmd, a), RZ_CMD_STATUS_OK, "pd was called correctly");
@@ -303,11 +291,11 @@ bool test_cmd_call_desc(void) {
 	mu_assert_eq(rz_cmd_call_parsed_args(cmd, a), RZ_CMD_STATUS_NONEXISTINGCMD, "px was not called because it does not exist");
 	rz_cmd_parsed_args_free(a);
 
-	a = rz_cmd_parsed_args_new("wv8", 1, wv8_args);
+	a = rz_cmd_parsed_args_new("wv", 1, wv_args);
 	mu_assert_eq(rz_cmd_call_parsed_args(cmd, a), RZ_CMD_STATUS_OK, "wv was called correctly");
 	rz_cmd_parsed_args_free(a);
 
-	a = rz_cmd_parsed_args_new("quit", 0, NULL);
+	a = rz_cmd_parsed_args_new("q", 0, NULL);
 	mu_assert_eq(rz_cmd_call_parsed_args(cmd, a), RZ_CMD_STATUS_EXIT, "quit is going to exit");
 	rz_cmd_parsed_args_free(a);
 
@@ -356,11 +344,11 @@ bool test_cmd_help(void) {
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &p_group_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
-	rz_cmd_desc_oldinput_new(cmd, p_cd, "px", px_handler, &px_help);
+	rz_cmd_desc_argv_new(cmd, p_cd, "px", px_handler, &px_help);
 
 	const char *p_help_exp = "Usage: p-usage   # p summary\n"
-				 "| pd <num>                    # pd summary\n"
-				 "| px[?] <verylongarg_str_num> # px summary\n";
+				 "| pd <num>                 # pd summary\n"
+				 "| px <verylongarg_str_num> # px summary\n";
 	RzCmdParsedArgs *a = rz_cmd_parsed_args_newcmd("p?");
 	char *h = rz_cmd_get_help(cmd, a, false);
 	mu_assert_notnull(h, "help is not null");
@@ -437,28 +425,6 @@ bool test_cmd_group_help(void) {
 	mu_end;
 }
 
-bool test_cmd_oldinput_help(void) {
-	rz_cons_new();
-
-	RzCmd *cmd = rz_cmd_new(NULL, true);
-	RzCmdDesc *root = rz_cmd_get_root(cmd);
-	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
-	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &fake_help);
-	rz_cmd_desc_oldinput_new(cmd, p_cd, "px", px_handler, NULL);
-
-	RzCmdParsedArgs *a = rz_cmd_parsed_args_newcmd("px?");
-	const char *px_help_exp = "Free format px help\n";
-	char *h = rz_cmd_get_help(cmd, a, false);
-	mu_assert_notnull(h, "help is not null");
-	mu_assert_streq(h, px_help_exp, "wrong help for px?");
-	free(h);
-	rz_cmd_parsed_args_free(a);
-
-	rz_cmd_free(cmd);
-	rz_cons_free();
-	mu_end;
-}
-
 bool test_cmd_group_exec_help(void) {
 	const RzCmdDescHelp p_help = {
 		.summary = "p summary",
@@ -741,10 +707,9 @@ bool test_foreach_cmdname(void) {
 	rz_cmd_desc_fake_new(cmd, root, "x", &fake_help);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pi", zd_handler, &fake_help);
-	RzCmdDesc *v_cd = rz_cmd_desc_oldinput_new(cmd, root, "v", a_oldinput_cb, &fake_help);
-	RzCmdDesc *v_inner_cd = rz_cmd_desc_inner_new(cmd, v_cd, "v", &fake_help);
-	rz_cmd_desc_argv_new(cmd, v_inner_cd, "v1", zd_handler, &fake_help);
-	rz_cmd_desc_argv_new(cmd, v_inner_cd, "v2", zd_handler, &fake_help);
+	RzCmdDesc *v_cd = rz_cmd_desc_group_new(cmd, root, "v", zd_handler, &fake_help, &fake_help);
+	rz_cmd_desc_argv_new(cmd, v_cd, "v1", zd_handler, &fake_help);
+	rz_cmd_desc_argv_new(cmd, v_cd, "v2", zd_handler, &fake_help);
 
 	RzList *res = rz_list_newf(free);
 	rz_cmd_foreach_cmdname(cmd, NULL, foreach_cmdname_cb, res);
@@ -782,10 +747,9 @@ bool test_foreach_cmdname_begin(void) {
 	rz_cmd_desc_fake_new(cmd, root, "x", &fake_help);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pi", zd_handler, &fake_help);
-	RzCmdDesc *v_cd = rz_cmd_desc_oldinput_new(cmd, root, "v", a_oldinput_cb, &fake_help);
-	RzCmdDesc *v_inner_cd = rz_cmd_desc_inner_new(cmd, v_cd, "v", &fake_help);
-	rz_cmd_desc_argv_new(cmd, v_inner_cd, "v1", zd_handler, &fake_help);
-	rz_cmd_desc_argv_new(cmd, v_inner_cd, "v2", zd_handler, &fake_help);
+	RzCmdDesc *v_cd = rz_cmd_desc_group_new(cmd, root, "v", zd_handler, &fake_help, &fake_help);
+	rz_cmd_desc_argv_new(cmd, v_cd, "v1", zd_handler, &fake_help);
+	rz_cmd_desc_argv_new(cmd, v_cd, "v2", zd_handler, &fake_help);
 
 	RzList *res = rz_list_newf(free);
 	rz_cmd_foreach_cmdname(cmd, v_cd, foreach_cmdname_cb, res);
@@ -958,18 +922,18 @@ bool test_get_arg(void) {
 	RzCmdDesc *z_cd = rz_cmd_desc_argv_new(cmd, root, "z", z_last_handler, &z_help);
 	RzCmdDesc *x_cd = rz_cmd_desc_argv_new(cmd, root, "x", x_array_handler, &x_help);
 
-	const RzCmdDescArg *a1 = rz_cmd_desc_get_arg(cmd, z_cd, 0);
+	const RzCmdDescArg *a1 = rz_cmd_desc_get_arg(z_cd, 0);
 	mu_assert_streq(a1->name, "a1", "0th arg of z is a1");
-	const RzCmdDescArg *a2 = rz_cmd_desc_get_arg(cmd, z_cd, 1);
+	const RzCmdDescArg *a2 = rz_cmd_desc_get_arg(z_cd, 1);
 	mu_assert_streq(a2->name, "a2", "1th arg of z is a2");
-	const RzCmdDescArg *an = rz_cmd_desc_get_arg(cmd, z_cd, 10);
+	const RzCmdDescArg *an = rz_cmd_desc_get_arg(z_cd, 10);
 	mu_assert_streq(an->name, "a2", "10th arg of z is a2");
 
-	const RzCmdDescArg *b1 = rz_cmd_desc_get_arg(cmd, x_cd, 0);
+	const RzCmdDescArg *b1 = rz_cmd_desc_get_arg(x_cd, 0);
 	mu_assert_streq(b1->name, "b1", "0th arg of x is b1");
-	const RzCmdDescArg *b2 = rz_cmd_desc_get_arg(cmd, x_cd, 1);
+	const RzCmdDescArg *b2 = rz_cmd_desc_get_arg(x_cd, 1);
 	mu_assert_streq(b2->name, "b2", "1th arg of x is b2");
-	const RzCmdDescArg *bn = rz_cmd_desc_get_arg(cmd, x_cd, 10);
+	const RzCmdDescArg *bn = rz_cmd_desc_get_arg(x_cd, 10);
 	mu_assert_null(bn, "10th arg of x does not exist");
 
 	rz_cmd_free(cmd);
@@ -1165,10 +1129,10 @@ bool test_state_output_concat_table(void) {
 	const char *s = rz_cons_get_buffer();
 	const char *exp = "val \n"
 			  "----\n"
-			  "33\n"
+			  " 33\n"
 			  "name   val \n"
 			  "-----------\n"
-			  "second 55\n";
+			  "second  55\n";
 	mu_assert_streq(s, exp, "tables output was printed");
 	rz_cons_pop();
 	rz_cmd_state_output_fini(&state);
@@ -1192,7 +1156,7 @@ bool test_state_output_concat_mix(void) {
 	const char *s = rz_cons_get_buffer();
 	const char *exp = "val \n"
 			  "----\n"
-			  "33\n"
+			  " 33\n"
 			  "second=55\n";
 	mu_assert_streq(s, exp, "table+quiet output was printed");
 	rz_cons_pop();
@@ -1328,9 +1292,9 @@ bool test_get_best_match(void) {
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	RzCmdDesc *ap_cd = rz_cmd_desc_group_new(cmd, a_cd, "ap", ap_handler, NULL, &fake_help);
 	RzCmdDesc *apd_cd = rz_cmd_desc_argv_new(cmd, ap_cd, "apd", ap_handler, &fake_help);
-	RzCmdDesc *ae_cd = rz_cmd_desc_oldinput_new(cmd, a_cd, "ae", ae_handler, NULL);
+	RzCmdDesc *ae_cd = rz_cmd_desc_argv_new(cmd, a_cd, "ae", ae_handler, NULL);
 	rz_cmd_desc_argv_new(cmd, ae_cd, "aeir", aeir_handler, &fake_help);
-	rz_cmd_desc_oldinput_new(cmd, root, "w", w_handler, NULL);
+	rz_cmd_desc_argv_new(cmd, root, "w", w_handler, NULL);
 
 	mu_assert_ptreq(rz_cmd_get_desc_best(cmd, "ap"), ap_cd, "ap should be best match for ap");
 	mu_assert_ptreq(rz_cmd_get_desc_best(cmd, "apn"), ap_cd, "ap should be best match for apn");
@@ -1467,6 +1431,7 @@ bool test_call_multiple_macros(void) {
 }
 
 int all_tests() {
+	rz_cons_new();
 	mu_run_test(test_parsed_args_noargs);
 	mu_run_test(test_parsed_args_onearg);
 	mu_run_test(test_parsed_args_args);
@@ -1475,14 +1440,12 @@ int all_tests() {
 	mu_run_test(test_parsed_args_newargs);
 	mu_run_test(test_cmd_descriptor_argv);
 	mu_run_test(test_cmd_descriptor_argv_nested);
-	mu_run_test(test_cmd_descriptor_oldinput);
 	mu_run_test(test_cmd_descriptor_tree);
 	mu_run_test(test_cmd_descriptor_group);
 	mu_run_test(test_cmd_get_desc);
 	mu_run_test(test_cmd_call_desc);
 	mu_run_test(test_cmd_help);
 	mu_run_test(test_cmd_group_help);
-	mu_run_test(test_cmd_oldinput_help);
 	mu_run_test(test_cmd_group_exec_help);
 	mu_run_test(test_remove_cmd);
 	mu_run_test(test_cmd_args);
